"""
Python module generated from Java source file org.joml.Matrix3x2dc

Java source file obtained from artifact joml version 1.10.8

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import *
from org.joml import *
from typing import Any, Callable, Iterable, Tuple


class Matrix3x2dc:
    """
    Interface to a read-only view of a 3x2 matrix of double-precision floats.

    Author(s)
    - Kai Burjack
    """

    def m00(self) -> float:
        """
        Return the value of the matrix element at column 0 and row 0.

        Returns
        - the value of the matrix element
        """
        ...


    def m01(self) -> float:
        """
        Return the value of the matrix element at column 0 and row 1.

        Returns
        - the value of the matrix element
        """
        ...


    def m10(self) -> float:
        """
        Return the value of the matrix element at column 1 and row 0.

        Returns
        - the value of the matrix element
        """
        ...


    def m11(self) -> float:
        """
        Return the value of the matrix element at column 1 and row 1.

        Returns
        - the value of the matrix element
        """
        ...


    def m20(self) -> float:
        """
        Return the value of the matrix element at column 2 and row 0.

        Returns
        - the value of the matrix element
        """
        ...


    def m21(self) -> float:
        """
        Return the value of the matrix element at column 2 and row 1.

        Returns
        - the value of the matrix element
        """
        ...


    def mul(self, right: "Matrix3x2dc", dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Multiply this matrix by the supplied `right` matrix by assuming a third row in
        both matrices of `(0, 0, 1)` and store the result in `dest`.
        
        If `M` is `this` matrix and `R` the `right` matrix,
        then the new matrix will be `M * R`. So when transforming a
        vector `v` with the new matrix by using `M * R * v`, the
        transformation of the right matrix will be applied first!

        Arguments
        - right: the right operand of the matrix multiplication
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulLocal(self, left: "Matrix3x2dc", dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Pre-multiply this matrix by the supplied `left` matrix and store the result in `dest`.
        
        If `M` is `this` matrix and `L` the `left` matrix,
        then the new matrix will be `L * M`. So when transforming a
        vector `v` with the new matrix by using `L * M * v`, the
        transformation of `this` matrix will be applied first!

        Arguments
        - left: the left operand of the matrix multiplication
        - dest: the destination matrix, which will hold the result

        Returns
        - dest
        """
        ...


    def determinant(self) -> float:
        """
        Return the determinant of this matrix.

        Returns
        - the determinant
        """
        ...


    def invert(self, dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Invert the `this` matrix by assuming a third row in this matrix of `(0, 0, 1)`
        and store the result in `dest`.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def translate(self, x: float, y: float, dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Apply a translation to this matrix by translating by the given number of units in x and y and store the result
        in `dest`.
        
        If `M` is `this` matrix and `T` the translation
        matrix, then the new matrix will be `M * T`. So when
        transforming a vector `v` with the new matrix by using
        `M * T * v`, the translation will be applied first!

        Arguments
        - x: the offset to translate in x
        - y: the offset to translate in y
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def translate(self, offset: "Vector2dc", dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Apply a translation to this matrix by translating by the given number of units in x and y, and
        store the result in `dest`.
        
        If `M` is `this` matrix and `T` the translation
        matrix, then the new matrix will be `M * T`. So when
        transforming a vector `v` with the new matrix by using
        `M * T * v`, the translation will be applied first!

        Arguments
        - offset: the offset to translate
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def translateLocal(self, offset: "Vector2dc", dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Pre-multiply a translation to this matrix by translating by the given number of
        units in x and y and store the result in `dest`.
        
        If `M` is `this` matrix and `T` the translation
        matrix, then the new matrix will be `T * M`. So when
        transforming a vector `v` with the new matrix by using
        `T * M * v`, the translation will be applied last!

        Arguments
        - offset: the number of units in x and y by which to translate
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def translateLocal(self, x: float, y: float, dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Pre-multiply a translation to this matrix by translating by the given number of
        units in x and y and store the result in `dest`.
        
        If `M` is `this` matrix and `T` the translation
        matrix, then the new matrix will be `T * M`. So when
        transforming a vector `v` with the new matrix by using
        `T * M * v`, the translation will be applied last!

        Arguments
        - x: the offset to translate in x
        - y: the offset to translate in y
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def get(self, dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Get the current values of `this` matrix and store them into
        `dest`.

        Arguments
        - dest: the destination matrix

        Returns
        - dest
        """
        ...


    def get(self, buffer: "DoubleBuffer") -> "DoubleBuffer":
        """
        Store this matrix in column-major order into the supplied DoubleBuffer at the current
        buffer DoubleBuffer.position() position.
        
        This method will not increment the position of the given DoubleBuffer.
        
        In order to specify the offset into the DoubleBuffer at which
        the matrix is stored, use .get(int, DoubleBuffer), taking
        the absolute position as parameter.

        Arguments
        - buffer: will receive the values of this matrix in column-major order at its current position

        Returns
        - the passed in buffer

        See
        - .get(int, DoubleBuffer)
        """
        ...


    def get(self, index: int, buffer: "DoubleBuffer") -> "DoubleBuffer":
        """
        Store this matrix in column-major order into the supplied DoubleBuffer starting at the specified
        absolute buffer position/index.
        
        This method will not increment the position of the given DoubleBuffer.

        Arguments
        - index: the absolute position into the DoubleBuffer
        - buffer: will receive the values of this matrix in column-major order

        Returns
        - the passed in buffer
        """
        ...


    def get(self, buffer: "ByteBuffer") -> "ByteBuffer":
        """
        Store this matrix in column-major order into the supplied ByteBuffer at the current
        buffer ByteBuffer.position() position.
        
        This method will not increment the position of the given ByteBuffer.
        
        In order to specify the offset into the ByteBuffer at which
        the matrix is stored, use .get(int, ByteBuffer), taking
        the absolute position as parameter.

        Arguments
        - buffer: will receive the values of this matrix in column-major order at its current position

        Returns
        - the passed in buffer

        See
        - .get(int, ByteBuffer)
        """
        ...


    def get(self, index: int, buffer: "ByteBuffer") -> "ByteBuffer":
        """
        Store this matrix in column-major order into the supplied ByteBuffer starting at the specified
        absolute buffer position/index.
        
        This method will not increment the position of the given ByteBuffer.

        Arguments
        - index: the absolute position into the ByteBuffer
        - buffer: will receive the values of this matrix in column-major order

        Returns
        - the passed in buffer
        """
        ...


    def getTransposed(self, buffer: "DoubleBuffer") -> "DoubleBuffer":
        """
        Store this matrix in row-major order into the supplied DoubleBuffer at the current
        buffer DoubleBuffer.position() position.
        
        This method will not increment the position of the given DoubleBuffer.
        
        In order to specify the offset into the DoubleBuffer at which
        the matrix is stored, use .getTransposed(int, DoubleBuffer), taking
        the absolute position as parameter.

        Arguments
        - buffer: will receive the values of this matrix in row-major order at its current position

        Returns
        - the passed in buffer

        See
        - .getTransposed(int, DoubleBuffer)
        """
        ...


    def getTransposed(self, index: int, buffer: "DoubleBuffer") -> "DoubleBuffer":
        """
        Store this matrix in row-major order into the supplied DoubleBuffer starting at the specified
        absolute buffer position/index.
        
        This method will not increment the position of the given DoubleBuffer.

        Arguments
        - index: the absolute position into the DoubleBuffer
        - buffer: will receive the values of this matrix in row-major order

        Returns
        - the passed in buffer
        """
        ...


    def getTransposed(self, buffer: "ByteBuffer") -> "ByteBuffer":
        """
        Store this matrix in row-major order into the supplied ByteBuffer at the current
        buffer ByteBuffer.position() position.
        
        This method will not increment the position of the given ByteBuffer.
        
        In order to specify the offset into the ByteBuffer at which
        the matrix is stored, use .getTransposed(int, ByteBuffer), taking
        the absolute position as parameter.

        Arguments
        - buffer: will receive the values of this matrix in row-major order at its current position

        Returns
        - the passed in buffer

        See
        - .getTransposed(int, ByteBuffer)
        """
        ...


    def getTransposed(self, index: int, buffer: "ByteBuffer") -> "ByteBuffer":
        """
        Store this matrix in row-major order into the supplied ByteBuffer starting at the specified
        absolute buffer position/index.
        
        This method will not increment the position of the given ByteBuffer.

        Arguments
        - index: the absolute position into the ByteBuffer
        - buffer: will receive the values of this matrix in row-major order

        Returns
        - the passed in buffer
        """
        ...


    def getTransposed(self, buffer: "FloatBuffer") -> "FloatBuffer":
        """
        Store this matrix in row-major order into the supplied FloatBuffer at the current
        buffer FloatBuffer.position() position.
        
        This method will not increment the position of the given FloatBuffer.
        
        Please note that due to this matrix storing double values those values will potentially
        lose precision when they are converted to float values before being put into the given FloatBuffer.
        
        In order to specify the offset into the FloatBuffer at which
        the matrix is stored, use .getTransposed(int, FloatBuffer), taking
        the absolute position as parameter.

        Arguments
        - buffer: will receive the values of this matrix in row-major order at its current position

        Returns
        - the passed in buffer

        See
        - .getTransposed(int, FloatBuffer)
        """
        ...


    def getTransposed(self, index: int, buffer: "FloatBuffer") -> "FloatBuffer":
        """
        Store this matrix in row-major order into the supplied FloatBuffer starting at the specified
        absolute buffer position/index.
        
        This method will not increment the position of the given FloatBuffer.
        
        Please note that due to this matrix storing double values those values will potentially
        lose precision when they are converted to float values before being put into the given FloatBuffer.

        Arguments
        - index: the absolute position into the FloatBuffer
        - buffer: will receive the values of this matrix in row-major order

        Returns
        - the passed in buffer
        """
        ...


    def getTransposedFloats(self, buffer: "ByteBuffer") -> "ByteBuffer":
        """
        Store this matrix as float values in row-major order into the supplied ByteBuffer at the current
        buffer ByteBuffer.position() position.
        
        This method will not increment the position of the given ByteBuffer.
        
        Please note that due to this matrix storing double values those values will potentially
        lose precision when they are converted to float values before being put into the given FloatBuffer.
        
        In order to specify the offset into the ByteBuffer at which
        the matrix is stored, use .getTransposedFloats(int, ByteBuffer), taking
        the absolute position as parameter.

        Arguments
        - buffer: will receive the values of this matrix as float values in row-major order at its current position

        Returns
        - the passed in buffer

        See
        - .getTransposedFloats(int, ByteBuffer)
        """
        ...


    def getTransposedFloats(self, index: int, buffer: "ByteBuffer") -> "ByteBuffer":
        """
        Store this matrix in row-major order into the supplied ByteBuffer starting at the specified
        absolute buffer position/index.
        
        This method will not increment the position of the given ByteBuffer.
        
        Please note that due to this matrix storing double values those values will potentially
        lose precision when they are converted to float values before being put into the given FloatBuffer.

        Arguments
        - index: the absolute position into the ByteBuffer
        - buffer: will receive the values of this matrix as float values in row-major order

        Returns
        - the passed in buffer
        """
        ...


    def get3x3(self, buffer: "DoubleBuffer") -> "DoubleBuffer":
        """
        Store this matrix as an equivalent 3x3 matrix in column-major order into the supplied DoubleBuffer at the current
        buffer DoubleBuffer.position() position.
        
        This method will not increment the position of the given DoubleBuffer.
        
        In order to specify the offset into the DoubleBuffer at which
        the matrix is stored, use .get3x3(int, DoubleBuffer), taking
        the absolute position as parameter.

        Arguments
        - buffer: will receive the values of this matrix in column-major order at its current position

        Returns
        - the passed in buffer

        See
        - .get3x3(int, DoubleBuffer)
        """
        ...


    def get3x3(self, index: int, buffer: "DoubleBuffer") -> "DoubleBuffer":
        """
        Store this matrix as an equivalent 3x3 matrix in column-major order into the supplied DoubleBuffer starting at the specified
        absolute buffer position/index.
        
        This method will not increment the position of the given DoubleBuffer.

        Arguments
        - index: the absolute position into the DoubleBuffer
        - buffer: will receive the values of this matrix in column-major order

        Returns
        - the passed in buffer
        """
        ...


    def get3x3(self, buffer: "ByteBuffer") -> "ByteBuffer":
        """
        Store this matrix as an equivalent 3x3 matrix in column-major order into the supplied ByteBuffer at the current
        buffer ByteBuffer.position() position.
        
        This method will not increment the position of the given ByteBuffer.
        
        In order to specify the offset into the ByteBuffer at which
        the matrix is stored, use .get3x3(int, ByteBuffer), taking
        the absolute position as parameter.

        Arguments
        - buffer: will receive the values of this matrix in column-major order at its current position

        Returns
        - the passed in buffer

        See
        - .get3x3(int, ByteBuffer)
        """
        ...


    def get3x3(self, index: int, buffer: "ByteBuffer") -> "ByteBuffer":
        """
        Store this matrix as an equivalent 3x3 matrix in column-major order into the supplied ByteBuffer starting at the specified
        absolute buffer position/index.
        
        This method will not increment the position of the given ByteBuffer.

        Arguments
        - index: the absolute position into the ByteBuffer
        - buffer: will receive the values of this matrix in column-major order

        Returns
        - the passed in buffer
        """
        ...


    def get4x4(self, buffer: "DoubleBuffer") -> "DoubleBuffer":
        """
        Store this matrix as an equivalent 4x4 matrix in column-major order into the supplied DoubleBuffer at the current
        buffer DoubleBuffer.position() position.
        
        This method will not increment the position of the given DoubleBuffer.
        
        In order to specify the offset into the DoubleBuffer at which
        the matrix is stored, use .get4x4(int, DoubleBuffer), taking
        the absolute position as parameter.

        Arguments
        - buffer: will receive the values of this matrix in column-major order at its current position

        Returns
        - the passed in buffer

        See
        - .get4x4(int, DoubleBuffer)
        """
        ...


    def get4x4(self, index: int, buffer: "DoubleBuffer") -> "DoubleBuffer":
        """
        Store this matrix as an equivalent 4x4 matrix in column-major order into the supplied DoubleBuffer starting at the specified
        absolute buffer position/index.
        
        This method will not increment the position of the given DoubleBuffer.

        Arguments
        - index: the absolute position into the DoubleBuffer
        - buffer: will receive the values of this matrix in column-major order

        Returns
        - the passed in buffer
        """
        ...


    def get4x4(self, buffer: "ByteBuffer") -> "ByteBuffer":
        """
        Store this matrix as an equivalent 4x4 matrix in column-major order into the supplied ByteBuffer at the current
        buffer ByteBuffer.position() position.
        
        This method will not increment the position of the given ByteBuffer.
        
        In order to specify the offset into the ByteBuffer at which
        the matrix is stored, use .get4x4(int, ByteBuffer), taking
        the absolute position as parameter.

        Arguments
        - buffer: will receive the values of this matrix in column-major order at its current position

        Returns
        - the passed in buffer

        See
        - .get4x4(int, ByteBuffer)
        """
        ...


    def get4x4(self, index: int, buffer: "ByteBuffer") -> "ByteBuffer":
        """
        Store this matrix as an equivalent 4x4 matrix in column-major order into the supplied ByteBuffer starting at the specified
        absolute buffer position/index.
        
        This method will not increment the position of the given ByteBuffer.

        Arguments
        - index: the absolute position into the ByteBuffer
        - buffer: will receive the values of this matrix in column-major order

        Returns
        - the passed in buffer
        """
        ...


    def getToAddress(self, address: int) -> "Matrix3x2dc":
        """
        Store this matrix in column-major order at the given off-heap address.
        
        This method will throw an UnsupportedOperationException when JOML is used with `-Djoml.nounsafe`.
        
        *This method is unsafe as it can result in a crash of the JVM process when the specified address range does not belong to this process.*

        Arguments
        - address: the off-heap address where to store this matrix

        Returns
        - this
        """
        ...


    def getTransposedToAddress(self, address: int) -> "Matrix3x2dc":
        """
        Store this matrix in row-major order at the given off-heap address.
        
        This method will throw an UnsupportedOperationException when JOML is used with `-Djoml.nounsafe`.
        
        *This method is unsafe as it can result in a crash of the JVM process when the specified address range does not belong to this process.*

        Arguments
        - address: the off-heap address where to store this matrix

        Returns
        - this
        """
        ...


    def get(self, arr: list[float], offset: int) -> list[float]:
        """
        Store this matrix into the supplied double array in column-major order at the given offset.

        Arguments
        - arr: the array to write the matrix values into
        - offset: the offset into the array

        Returns
        - the passed in array
        """
        ...


    def get(self, arr: list[float]) -> list[float]:
        """
        Store this matrix into the supplied double array in column-major order.
        
        In order to specify an explicit offset into the array, use the method .get(double[], int).

        Arguments
        - arr: the array to write the matrix values into

        Returns
        - the passed in array

        See
        - .get(double[], int)
        """
        ...


    def get3x3(self, arr: list[float], offset: int) -> list[float]:
        """
        Store this matrix as an equivalent 3x3 matrix into the supplied double array in column-major order at the given offset.

        Arguments
        - arr: the array to write the matrix values into
        - offset: the offset into the array

        Returns
        - the passed in array
        """
        ...


    def get3x3(self, arr: list[float]) -> list[float]:
        """
        Store this matrix as an equivalent 3x3 matrix into the supplied double array in column-major order.
        
        In order to specify an explicit offset into the array, use the method .get3x3(double[], int).

        Arguments
        - arr: the array to write the matrix values into

        Returns
        - the passed in array

        See
        - .get3x3(double[], int)
        """
        ...


    def get4x4(self, arr: list[float], offset: int) -> list[float]:
        """
        Store this matrix as an equivalent 4x4 matrix into the supplied double array in column-major order at the given offset.

        Arguments
        - arr: the array to write the matrix values into
        - offset: the offset into the array

        Returns
        - the passed in array
        """
        ...


    def get4x4(self, arr: list[float]) -> list[float]:
        """
        Store this matrix as an equivalent 4x4 matrix into the supplied double array in column-major order.
        
        In order to specify an explicit offset into the array, use the method .get4x4(double[], int).

        Arguments
        - arr: the array to write the matrix values into

        Returns
        - the passed in array

        See
        - .get4x4(double[], int)
        """
        ...


    def scale(self, x: float, y: float, dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Apply scaling to this matrix by scaling the unit axes by the given x and y and store the result in `dest`.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `M * S`. So when transforming a
        vector `v` with the new matrix by using `M * S * v`, the scaling will be applied first!

        Arguments
        - x: the factor of the x component
        - y: the factor of the y component
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def scale(self, xy: "Vector2dc", dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Apply scaling to this matrix by scaling the base axes by the given `xy` factors
        and store the result in `dest`.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `M * S`. So when transforming a
        vector `v` with the new matrix by using `M * S * v`, the scaling will be applied first!

        Arguments
        - xy: the factors of the x and y component, respectively
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def scale(self, xy: "Vector2fc", dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Apply scaling to this matrix by scaling the base axes by the given `xy` factors
        and store the result in `dest`.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `M * S`. So when transforming a
        vector `v` with the new matrix by using `M * S * v`, the scaling will be applied first!

        Arguments
        - xy: the factors of the x and y component, respectively
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def scaleLocal(self, xy: float, dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Pre-multiply scaling to `this` matrix by scaling the two base axes by the given `xy` factor,
        and store the result in `dest`.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `S * M`. So when transforming a
        vector `v` with the new matrix by using `S * M * v`
        , the scaling will be applied last!

        Arguments
        - xy: the factor to scale all two base axes by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def scaleLocal(self, x: float, y: float, dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Pre-multiply scaling to `this` matrix by scaling the base axes by the given x and y
        factors and store the result in `dest`.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `S * M`. So when transforming a
        vector `v` with the new matrix by using `S * M * v`
        , the scaling will be applied last!

        Arguments
        - x: the factor of the x component
        - y: the factor of the y component
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def scaleAroundLocal(self, sx: float, sy: float, ox: float, oy: float, dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Pre-multiply scaling to `this` matrix by scaling the base axes by the given sx and
        sy factors while using the given `(ox, oy)` as the scaling origin,
        and store the result in `dest`.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `S * M`. So when transforming a
        vector `v` with the new matrix by using `S * M * v`
        , the scaling will be applied last!
        
        This method is equivalent to calling: `new Matrix3x2d().translate(ox, oy).scale(sx, sy).translate(-ox, -oy).mul(this, dest)`

        Arguments
        - sx: the scaling factor of the x component
        - sy: the scaling factor of the y component
        - ox: the x coordinate of the scaling origin
        - oy: the y coordinate of the scaling origin
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def scaleAroundLocal(self, factor: float, ox: float, oy: float, dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Pre-multiply scaling to this matrix by scaling the base axes by the given `factor`
        while using `(ox, oy)` as the scaling origin,
        and store the result in `dest`.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `S * M`. So when transforming a
        vector `v` with the new matrix by using `S * M * v`, the
        scaling will be applied last!
        
        This method is equivalent to calling: `new Matrix3x2d().translate(ox, oy).scale(factor).translate(-ox, -oy).mul(this, dest)`

        Arguments
        - factor: the scaling factor for all three axes
        - ox: the x coordinate of the scaling origin
        - oy: the y coordinate of the scaling origin
        - dest: will hold the result

        Returns
        - this
        """
        ...


    def scale(self, xy: float, dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Apply scaling to this matrix by uniformly scaling the two base axes by the given `xy` factor
        and store the result in `dest`.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `M * S`. So when transforming a
        vector `v` with the new matrix by using `M * S * v`, the scaling will be applied first!

        Arguments
        - xy: the factor for the two components
        - dest: will hold the result

        Returns
        - dest

        See
        - .scale(double, double, Matrix3x2d)
        """
        ...


    def scaleAround(self, sx: float, sy: float, ox: float, oy: float, dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Apply scaling to `this` matrix by scaling the base axes by the given sx and
        sy factors while using `(ox, oy)` as the scaling origin, and store the result in `dest`.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `M * S`. So when transforming a
        vector `v` with the new matrix by using `M * S * v`
        , the scaling will be applied first!
        
        This method is equivalent to calling: `translate(ox, oy, dest).scale(sx, sy).translate(-ox, -oy)`

        Arguments
        - sx: the scaling factor of the x component
        - sy: the scaling factor of the y component
        - ox: the x coordinate of the scaling origin
        - oy: the y coordinate of the scaling origin
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def scaleAround(self, factor: float, ox: float, oy: float, dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Apply scaling to this matrix by scaling the base axes by the given `factor`
        while using `(ox, oy)` as the scaling origin,
        and store the result in `dest`.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `M * S`. So when transforming a
        vector `v` with the new matrix by using `M * S * v`, the
        scaling will be applied first!
        
        This method is equivalent to calling: `translate(ox, oy, dest).scale(factor).translate(-ox, -oy)`

        Arguments
        - factor: the scaling factor for all three axes
        - ox: the x coordinate of the scaling origin
        - oy: the y coordinate of the scaling origin
        - dest: will hold the result

        Returns
        - this
        """
        ...


    def transform(self, v: "Vector3d") -> "Vector3d":
        """
        Transform/multiply the given vector by this matrix by assuming a third row in this matrix of `(0, 0, 1)`
        and store the result in that vector.

        Arguments
        - v: the vector to transform and to hold the final result

        Returns
        - v

        See
        - Vector3d.mul(Matrix3x2dc)
        """
        ...


    def transform(self, v: "Vector3dc", dest: "Vector3d") -> "Vector3d":
        """
        Transform/multiply the given vector by this matrix and store the result in `dest`.

        Arguments
        - v: the vector to transform
        - dest: will contain the result

        Returns
        - dest

        See
        - Vector3d.mul(Matrix3x2dc, Vector3d)
        """
        ...


    def transform(self, x: float, y: float, z: float, dest: "Vector3d") -> "Vector3d":
        """
        Transform/multiply the given vector `(x, y, z)` by this matrix and store the result in `dest`.

        Arguments
        - x: the x component of the vector to transform
        - y: the y component of the vector to transform
        - z: the z component of the vector to transform
        - dest: will contain the result

        Returns
        - dest
        """
        ...


    def transformPosition(self, v: "Vector2d") -> "Vector2d":
        """
        Transform/multiply the given 2D-vector, as if it was a 3D-vector with z=1, by
        this matrix and store the result in that vector.
        
        The given 2D-vector is treated as a 3D-vector with its z-component being 1.0, so it
        will represent a position/location in 2D-space rather than a direction.
        
        In order to store the result in another vector, use .transformPosition(Vector2dc, Vector2d).

        Arguments
        - v: the vector to transform and to hold the final result

        Returns
        - v

        See
        - .transform(Vector3d)
        """
        ...


    def transformPosition(self, v: "Vector2dc", dest: "Vector2d") -> "Vector2d":
        """
        Transform/multiply the given 2D-vector, as if it was a 3D-vector with z=1, by
        this matrix and store the result in `dest`.
        
        The given 2D-vector is treated as a 3D-vector with its z-component being 1.0, so it
        will represent a position/location in 2D-space rather than a direction.
        
        In order to store the result in the same vector, use .transformPosition(Vector2d).

        Arguments
        - v: the vector to transform
        - dest: will hold the result

        Returns
        - dest

        See
        - .transform(Vector3dc, Vector3d)
        """
        ...


    def transformPosition(self, x: float, y: float, dest: "Vector2d") -> "Vector2d":
        """
        Transform/multiply the given 2D-vector `(x, y)`, as if it was a 3D-vector with z=1, by
        this matrix and store the result in `dest`.
        
        The given 2D-vector is treated as a 3D-vector with its z-component being 1.0, so it
        will represent a position/location in 2D-space rather than a direction.
        
        In order to store the result in the same vector, use .transformPosition(Vector2d).

        Arguments
        - x: the x component of the vector to transform
        - y: the y component of the vector to transform
        - dest: will hold the result

        Returns
        - dest

        See
        - .transform(Vector3dc, Vector3d)
        """
        ...


    def transformDirection(self, v: "Vector2d") -> "Vector2d":
        """
        Transform/multiply the given 2D-vector, as if it was a 3D-vector with z=0, by
        this matrix and store the result in that vector.
        
        The given 2D-vector is treated as a 3D-vector with its z-component being `0.0`, so it
        will represent a direction in 2D-space rather than a position. This method will therefore
        not take the translation part of the matrix into account.
        
        In order to store the result in another vector, use .transformDirection(Vector2dc, Vector2d).

        Arguments
        - v: the vector to transform and to hold the final result

        Returns
        - v

        See
        - .transformDirection(Vector2dc, Vector2d)
        """
        ...


    def transformDirection(self, v: "Vector2dc", dest: "Vector2d") -> "Vector2d":
        """
        Transform/multiply the given 2D-vector, as if it was a 3D-vector with z=0, by
        this matrix and store the result in `dest`.
        
        The given 2D-vector is treated as a 3D-vector with its z-component being `0.0`, so it
        will represent a direction in 2D-space rather than a position. This method will therefore
        not take the translation part of the matrix into account.
        
        In order to store the result in the same vector, use .transformDirection(Vector2d).

        Arguments
        - v: the vector to transform and to hold the final result
        - dest: will hold the result

        Returns
        - dest

        See
        - .transformDirection(Vector2d)
        """
        ...


    def transformDirection(self, x: float, y: float, dest: "Vector2d") -> "Vector2d":
        """
        Transform/multiply the given 2D-vector `(x, y)`, as if it was a 3D-vector with z=0, by
        this matrix and store the result in `dest`.
        
        The given 2D-vector is treated as a 3D-vector with its z-component being `0.0`, so it
        will represent a direction in 2D-space rather than a position. This method will therefore
        not take the translation part of the matrix into account.
        
        In order to store the result in the same vector, use .transformDirection(Vector2d).

        Arguments
        - x: the x component of the vector to transform
        - y: the y component of the vector to transform
        - dest: will hold the result

        Returns
        - dest

        See
        - .transformDirection(Vector2d)
        """
        ...


    def rotate(self, ang: float, dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Apply a rotation transformation to this matrix by rotating the given amount of radians and store the result in `dest`.
        
        If `M` is `this` matrix and `R` the rotation matrix,
        then the new matrix will be `M * R`. So when transforming a
        vector `v` with the new matrix by using `M * R * v`, the rotation will be applied first!

        Arguments
        - ang: the angle in radians
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def rotateLocal(self, ang: float, dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Pre-multiply a rotation to this matrix by rotating the given amount of radians and store the result in `dest`.
        
        If `M` is `this` matrix and `R` the rotation matrix,
        then the new matrix will be `R * M`. So when transforming a
        vector `v` with the new matrix by using `R * M * v`, the
        rotation will be applied last!
        
        Reference: <a href="http://en.wikipedia.org/wiki/Rotation_matrix#Rotation_matrix_from_axis_and_angle">http://en.wikipedia.org</a>

        Arguments
        - ang: the angle in radians
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def rotateAbout(self, ang: float, x: float, y: float, dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Apply a rotation transformation to this matrix by rotating the given amount of radians about
        the specified rotation center `(x, y)` and store the result in `dest`.
        
        This method is equivalent to calling: `translate(x, y, dest).rotate(ang).translate(-x, -y)`
        
        If `M` is `this` matrix and `R` the rotation matrix,
        then the new matrix will be `M * R`. So when transforming a
        vector `v` with the new matrix by using `M * R * v`, the rotation will be applied first!

        Arguments
        - ang: the angle in radians
        - x: the x component of the rotation center
        - y: the y component of the rotation center
        - dest: will hold the result

        Returns
        - dest

        See
        - .rotate(double, Matrix3x2d)
        """
        ...


    def rotateTo(self, fromDir: "Vector2dc", toDir: "Vector2dc", dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Apply a rotation transformation to this matrix that rotates the given normalized `fromDir` direction vector
        to point along the normalized `toDir`, and store the result in `dest`.
        
        If `M` is `this` matrix and `R` the rotation matrix,
        then the new matrix will be `M * R`. So when transforming a
        vector `v` with the new matrix by using `M * R * v`, the rotation will be applied first!

        Arguments
        - fromDir: the normalized direction which should be rotate to point along `toDir`
        - toDir: the normalized destination direction
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def view(self, left: float, right: float, bottom: float, top: float, dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Apply a "view" transformation to this matrix that maps the given `(left, bottom)` and
        `(right, top)` corners to `(-1, -1)` and `(1, 1)` respectively and store the result in `dest`.
        
        If `M` is `this` matrix and `O` the orthographic projection matrix,
        then the new matrix will be `M * O`. So when transforming a
        vector `v` with the new matrix by using `M * O * v`, the
        orthographic projection transformation will be applied first!

        Arguments
        - left: the distance from the center to the left view edge
        - right: the distance from the center to the right view edge
        - bottom: the distance from the center to the bottom view edge
        - top: the distance from the center to the top view edge
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def origin(self, origin: "Vector2d") -> "Vector2d":
        """
        Obtain the position that gets transformed to the origin by `this` matrix.
        This can be used to get the position of the "camera" from a given *view* transformation matrix.
        
        This method is equivalent to the following code:
        ```
        Matrix3x2d inv = new Matrix3x2d(this).invertAffine();
        inv.transform(origin.set(0, 0));
        ```

        Arguments
        - origin: will hold the position transformed to the origin

        Returns
        - origin
        """
        ...


    def viewArea(self, area: list[float]) -> list[float]:
        """
        Obtain the extents of the view transformation of `this` matrix and store it in `area`.
        This can be used to determine which region of the screen (i.e. the NDC space) is covered by the view.

        Arguments
        - area: will hold the view area as `[minX, minY, maxX, maxY]`

        Returns
        - area
        """
        ...


    def positiveX(self, dir: "Vector2d") -> "Vector2d":
        """
        Obtain the direction of `+X` before the transformation represented by `this` matrix is applied.
        
        This method uses the rotation component of the left 2x2 submatrix to obtain the direction 
        that is transformed to `+X` by `this` matrix.
        
        This method is equivalent to the following code:
        ```
        Matrix3x2d inv = new Matrix3x2d(this).invert();
        inv.transformDirection(dir.set(1, 0)).normalize();
        ```
        If `this` is already an orthogonal matrix, then consider using .normalizedPositiveX(Vector2d) instead.
        
        Reference: <a href="http://www.euclideanspace.com/maths/algebra/matrix/functions/inverse/threeD/">http://www.euclideanspace.com</a>

        Arguments
        - dir: will hold the direction of `+X`

        Returns
        - dir
        """
        ...


    def normalizedPositiveX(self, dir: "Vector2d") -> "Vector2d":
        """
        Obtain the direction of `+X` before the transformation represented by `this` *orthogonal* matrix is applied.
        This method only produces correct results if `this` is an *orthogonal* matrix.
        
        This method uses the rotation component of the left 2x2 submatrix to obtain the direction 
        that is transformed to `+X` by `this` matrix.
        
        This method is equivalent to the following code:
        ```
        Matrix3x2d inv = new Matrix3x2d(this).transpose();
        inv.transformDirection(dir.set(1, 0));
        ```
        
        Reference: <a href="http://www.euclideanspace.com/maths/algebra/matrix/functions/inverse/threeD/">http://www.euclideanspace.com</a>

        Arguments
        - dir: will hold the direction of `+X`

        Returns
        - dir
        """
        ...


    def positiveY(self, dir: "Vector2d") -> "Vector2d":
        """
        Obtain the direction of `+Y` before the transformation represented by `this` matrix is applied.
        
        This method uses the rotation component of the left 2x2 submatrix to obtain the direction 
        that is transformed to `+Y` by `this` matrix.
        
        This method is equivalent to the following code:
        ```
        Matrix3x2d inv = new Matrix3x2d(this).invert();
        inv.transformDirection(dir.set(0, 1)).normalize();
        ```
        If `this` is already an orthogonal matrix, then consider using .normalizedPositiveY(Vector2d) instead.
        
        Reference: <a href="http://www.euclideanspace.com/maths/algebra/matrix/functions/inverse/threeD/">http://www.euclideanspace.com</a>

        Arguments
        - dir: will hold the direction of `+Y`

        Returns
        - dir
        """
        ...


    def normalizedPositiveY(self, dir: "Vector2d") -> "Vector2d":
        """
        Obtain the direction of `+Y` before the transformation represented by `this` *orthogonal* matrix is applied.
        This method only produces correct results if `this` is an *orthogonal* matrix.
        
        This method uses the rotation component of the left 2x2 submatrix to obtain the direction 
        that is transformed to `+Y` by `this` matrix.
        
        This method is equivalent to the following code:
        ```
        Matrix3x2d inv = new Matrix3x2d(this).transpose();
        inv.transformDirection(dir.set(0, 1));
        ```
        
        Reference: <a href="http://www.euclideanspace.com/maths/algebra/matrix/functions/inverse/threeD/">http://www.euclideanspace.com</a>

        Arguments
        - dir: will hold the direction of `+Y`

        Returns
        - dir
        """
        ...


    def unproject(self, winX: float, winY: float, viewport: list[int], dest: "Vector2d") -> "Vector2d":
        """
        Unproject the given window coordinates `(winX, winY)` by `this` matrix using the specified viewport.
        
        This method first converts the given window coordinates to normalized device coordinates in the range `[-1..1]`
        and then transforms those NDC coordinates by the inverse of `this` matrix.  
        
        As a necessary computation step for unprojecting, this method computes the inverse of `this` matrix.
        In order to avoid computing the matrix inverse with every invocation, the inverse of `this` matrix can be built
        once outside using .invert(Matrix3x2d) and then the method .unprojectInv(double, double, int[], Vector2d) unprojectInv() can be invoked on it.

        Arguments
        - winX: the x-coordinate in window coordinates (pixels)
        - winY: the y-coordinate in window coordinates (pixels)
        - viewport: the viewport described by `[x, y, width, height]`
        - dest: will hold the unprojected position

        Returns
        - dest

        See
        - .invert(Matrix3x2d)
        """
        ...


    def unprojectInv(self, winX: float, winY: float, viewport: list[int], dest: "Vector2d") -> "Vector2d":
        """
        Unproject the given window coordinates `(winX, winY)` by `this` matrix using the specified viewport.
        
        This method differs from .unproject(double, double, int[], Vector2d) unproject() 
        in that it assumes that `this` is already the inverse matrix of the original projection matrix.
        It exists to avoid recomputing the matrix inverse with every invocation.

        Arguments
        - winX: the x-coordinate in window coordinates (pixels)
        - winY: the y-coordinate in window coordinates (pixels)
        - viewport: the viewport described by `[x, y, width, height]`
        - dest: will hold the unprojected position

        Returns
        - dest

        See
        - .unproject(double, double, int[], Vector2d)
        """
        ...


    def testPoint(self, x: float, y: float) -> bool:
        """
        Test whether the given point `(x, y)` is within the frustum defined by `this` matrix.
        
        This method assumes `this` matrix to be a transformation from any arbitrary coordinate system/space `M`
        into standard OpenGL clip space and tests whether the given point with the coordinates `(x, y, z)` given
        in space `M` is within the clip space.
        
        Reference: <a href="http://gamedevs.org/uploads/fast-extraction-viewing-frustum-planes-from-world-view-projection-matrix.pdf">
        Fast Extraction of Viewing Frustum Planes from the World-View-Projection Matrix</a>

        Arguments
        - x: the x-coordinate of the point
        - y: the y-coordinate of the point

        Returns
        - `True` if the given point is inside the frustum; `False` otherwise
        """
        ...


    def testCircle(self, x: float, y: float, r: float) -> bool:
        """
        Test whether the given circle is partly or completely within or outside of the frustum defined by `this` matrix.
        
        This method assumes `this` matrix to be a transformation from any arbitrary coordinate system/space `M`
        into standard OpenGL clip space and tests whether the given sphere with the coordinates `(x, y, z)` given
        in space `M` is within the clip space.
        
        Reference: <a href="http://gamedevs.org/uploads/fast-extraction-viewing-frustum-planes-from-world-view-projection-matrix.pdf">
        Fast Extraction of Viewing Frustum Planes from the World-View-Projection Matrix</a>

        Arguments
        - x: the x-coordinate of the circle's center
        - y: the y-coordinate of the circle's center
        - r: the circle's radius

        Returns
        - `True` if the given circle is partly or completely inside the frustum; `False` otherwise
        """
        ...


    def testAar(self, minX: float, minY: float, maxX: float, maxY: float) -> bool:
        """
        Test whether the given axis-aligned rectangle is partly or completely within or outside of the frustum defined by `this` matrix.
        The rectangle is specified via its min and max corner coordinates.
        
        This method assumes `this` matrix to be a transformation from any arbitrary coordinate system/space `M`
        into standard OpenGL clip space and tests whether the given axis-aligned rectangle with its minimum corner coordinates `(minX, minY, minZ)`
        and maximum corner coordinates `(maxX, maxY, maxZ)` given in space `M` is within the clip space.
        
        Reference: <a href="http://old.cescg.org/CESCG-2002/DSykoraJJelinek/">Efficient View Frustum Culling</a>
        
        Reference: <a href="http://gamedevs.org/uploads/fast-extraction-viewing-frustum-planes-from-world-view-projection-matrix.pdf">
        Fast Extraction of Viewing Frustum Planes from the World-View-Projection Matrix</a>

        Arguments
        - minX: the x-coordinate of the minimum corner
        - minY: the y-coordinate of the minimum corner
        - maxX: the x-coordinate of the maximum corner
        - maxY: the y-coordinate of the maximum corner

        Returns
        - `True` if the axis-aligned box is completely or partly inside of the frustum; `False` otherwise
        """
        ...


    def equals(self, m: "Matrix3x2dc", delta: float) -> bool:
        """
        Compare the matrix elements of `this` matrix with the given matrix using the given `delta`
        and return whether all of them are equal within a maximum difference of `delta`.
        
        Please note that this method is not used by any data structure such as ArrayList HashSet or HashMap
        and their operations, such as ArrayList.contains(Object) or HashSet.remove(Object), since those
        data structures only use the Object.equals(Object) and Object.hashCode() methods.

        Arguments
        - m: the other matrix
        - delta: the allowed maximum difference

        Returns
        - `True` whether all of the matrix elements are equal; `False` otherwise
        """
        ...


    def isFinite(self) -> bool:
        """
        Determine whether all matrix elements are finite floating-point values, that
        is, they are not Double.isNaN() NaN and not
        Double.isInfinite() infinity.

        Returns
        - `True` if all components are finite floating-point values;
                `False` otherwise
        """
        ...



"""
Python module generated from Java source file org.bukkit.util.noise.PerlinOctaveGenerator

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import Random
from org.bukkit import World
from org.bukkit.util.noise import *
from typing import Any, Callable, Iterable, Tuple


class PerlinOctaveGenerator(OctaveGenerator):
    """
    Creates perlin noise through unbiased octaves
    """

    def __init__(self, world: "World", octaves: int):
        """
        Creates a perlin octave generator for the given world

        Arguments
        - world: World to construct this generator for
        - octaves: Amount of octaves to create
        """
        ...


    def __init__(self, seed: int, octaves: int):
        """
        Creates a perlin octave generator for the given world

        Arguments
        - seed: Seed to construct this generator for
        - octaves: Amount of octaves to create
        """
        ...


    def __init__(self, rand: "Random", octaves: int):
        """
        Creates a perlin octave generator for the given Random

        Arguments
        - rand: Random object to construct this generator for
        - octaves: Amount of octaves to create
        """
        ...



"""
Python module generated from Java source file org.bukkit.inventory.view.EnchantmentView

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.enchantments import EnchantmentOffer
from org.bukkit.inventory import EnchantingInventory
from org.bukkit.inventory import InventoryView
from org.bukkit.inventory.view import *
from typing import Any, Callable, Iterable, Tuple


class EnchantmentView(InventoryView):
    """
    An instance of InventoryView which provides extra methods related to
    enchantment table view data.
    """

    def getTopInventory(self) -> "EnchantingInventory":
        ...


    def getEnchantmentSeed(self) -> int:
        """
        Gets the random enchantment seed used in this view

        Returns
        - The random seed used
        """
        ...


    def getOffers(self) -> list["EnchantmentOffer"]:
        """
        Gets the offers of this EnchantmentView

        Returns
        - The enchantment offers that are provided
        """
        ...


    def setOffers(self, offers: list["EnchantmentOffer"]) -> None:
        """
        Sets the offers to provide to the player.

        Arguments
        - offers: The offers to provide

        Raises
        - IllegalArgumentException: if the array length isn't 3
        """
        ...



"""
Python module generated from Java source file org.bukkit.inventory.meta.WritableBookMeta

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Material
from org.bukkit.inventory.meta import *
from typing import Any, Callable, Iterable, Tuple


class WritableBookMeta(ItemMeta):
    """
    Represents a book (Material.WRITABLE_BOOK or Material.WRITTEN_BOOK) that can have pages.
    """

    def hasPages(self) -> bool:
        """
        Checks for the existence of pages in the book.

        Returns
        - True if the book has pages
        """
        ...


    def getPage(self, page: int) -> str:
        """
        Gets the specified page in the book. The given page must exist.
        
        Pages are 1-indexed.

        Arguments
        - page: the page number to get, in range [1, getPageCount()]

        Returns
        - the page from the book
        """
        ...


    def setPage(self, page: int, data: str) -> None:
        """
        Sets the specified page in the book. Pages of the book must be
        contiguous.
        
        The data can be up to 1024 characters in length, additional characters
        are truncated.
        
        Pages are 1-indexed.

        Arguments
        - page: the page number to set, in range [1, getPageCount()]
        - data: the data to set for that page
        """
        ...


    def getPages(self) -> list[str]:
        """
        Gets all the pages in the book.

        Returns
        - list of all the pages in the book
        """
        ...


    def setPages(self, pages: list[str]) -> None:
        """
        Clears the existing book pages, and sets the book to use the provided
        pages. Maximum 100 pages with 1024 characters per page.

        Arguments
        - pages: A list of pages to set the book to use
        """
        ...


    def setPages(self, *pages: Tuple[str, ...]) -> None:
        """
        Clears the existing book pages, and sets the book to use the provided
        pages. Maximum 100 pages with 1024 characters per page.

        Arguments
        - pages: A list of strings, each being a page
        """
        ...


    def addPage(self, *pages: Tuple[str, ...]) -> None:
        """
        Adds new pages to the end of the book. Up to a maximum of 100 pages with
        1024 characters per page.

        Arguments
        - pages: A list of strings, each being a page
        """
        ...


    def getPageCount(self) -> int:
        """
        Gets the number of pages in the book.

        Returns
        - the number of pages in the book
        """
        ...


    def clone(self) -> "WritableBookMeta":
        ...



"""
Python module generated from Java source file com.google.common.util.concurrent.ForwardingExecutorService

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.collect import ForwardingObject
from com.google.common.util.concurrent import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.errorprone.annotations import CheckReturnValue
from java.util.concurrent import Callable
from java.util.concurrent import ExecutionException
from java.util.concurrent import ExecutorService
from java.util.concurrent import Future
from java.util.concurrent import TimeUnit
from java.util.concurrent import TimeoutException
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class ForwardingExecutorService(ForwardingObject, ExecutorService):
    """
    An executor service which forwards all its method calls to another executor service. Subclasses
    should override one or more methods to modify the behavior of the backing executor service as
    desired per the <a href="http://en.wikipedia.org/wiki/Decorator_pattern">decorator pattern</a>.
    
    **`default` method warning:** This class does *not* forward calls to `default` methods. Instead, it inherits their default implementations. When those implementations
    invoke methods, they invoke methods on the `ForwardingExecutorService`.

    Author(s)
    - Kurt Alfred Kluever

    Since
    - 10.0
    """

    def awaitTermination(self, timeout: int, unit: "TimeUnit") -> bool:
        ...


    def invokeAll(self, tasks: Iterable["Callable"["T"]]) -> list["Future"["T"]]:
        ...


    def invokeAll(self, tasks: Iterable["Callable"["T"]], timeout: int, unit: "TimeUnit") -> list["Future"["T"]]:
        ...


    def invokeAny(self, tasks: Iterable["Callable"["T"]]) -> "T":
        ...


    def invokeAny(self, tasks: Iterable["Callable"["T"]], timeout: int, unit: "TimeUnit") -> "T":
        ...


    def isShutdown(self) -> bool:
        ...


    def isTerminated(self) -> bool:
        ...


    def shutdown(self) -> None:
        ...


    def shutdownNow(self) -> list["Runnable"]:
        ...


    def execute(self, command: "Runnable") -> None:
        ...


    def submit(self, task: "Callable"["T"]) -> "Future"["T"]:
        ...


    def submit(self, task: "Runnable") -> "Future"[Any]:
        ...


    def submit(self, task: "Runnable", result: "T") -> "Future"["T"]:
        ...



"""
Python module generated from Java source file com.google.common.collect.Multiset

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.errorprone.annotations import CompatibleWith
from java.util import Collections
from java.util import Iterator
from java.util import Spliterator
from java.util.function import Consumer
from java.util.function import ObjIntConsumer
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class Multiset(Collection):
    """
    A collection that supports order-independent equality, like Set, but may have duplicate
    elements. A multiset is also sometimes called a *bag*.
    
    Elements of a multiset that are equal to one another are referred to as *occurrences* of
    the same single element. The total number of occurrences of an element in a multiset is called
    the *count* of that element (the terms "frequency" and "multiplicity" are equivalent, but
    not used in this API). Since the count of an element is represented as an `int`, a multiset
    may never contain more than Integer.MAX_VALUE occurrences of any one element.
    
    `Multiset` refines the specifications of several methods from `Collection`. It
    also defines an additional query operation, .count, which returns the count of an
    element. There are five new bulk-modification operations, for example .add(Object, int),
    to add or remove multiple occurrences of an element at once, or to set the count of an element to
    a specific value. These modification operations are optional, but implementations which support
    the standard collection operations .add(Object) or .remove(Object) are encouraged
    to implement the related methods as well. Finally, two collection views are provided: .elementSet contains the distinct elements of the multiset "with duplicates collapsed", and
    .entrySet is similar but contains Entry Multiset.Entry instances, each providing
    both a distinct element and the count of that element.
    
    In addition to these required methods, implementations of `Multiset` are expected to
    provide two `static` creation methods: `create()`, returning an empty multiset, and
    `create(Iterable<? extends E>)`, returning a multiset containing the given initial
    elements. This is simply a refinement of `Collection`'s constructor recommendations,
    reflecting the new developments of Java 5.
    
    As with other collection types, the modification operations are optional, and should throw
    UnsupportedOperationException when they are not implemented. Most implementations should
    support either all add operations or none of them, all removal operations or none of them, and if
    and only if all of these are supported, the `setCount` methods as well.
    
    A multiset uses Object.equals to determine whether two instances should be considered
    "the same," *unless specified otherwise* by the implementation.
    
    **Warning:** as with normal Sets, it is almost always a bad idea to modify an
    element (in a way that affects its Object.equals behavior) while it is contained in a
    multiset. Undefined behavior and bugs will result.
    
    <h3>Implementations</h3>
    
    
      - ImmutableMultiset
      - ImmutableSortedMultiset
      - HashMultiset
      - LinkedHashMultiset
      - TreeMultiset
      - EnumMultiset
      - ConcurrentHashMultiset
    
    
    If your values may be zero, negative, or outside the range of an int, you may wish to use
    com.google.common.util.concurrent.AtomicLongMap instead. Note, however, that unlike
    `Multiset`, `AtomicLongMap` does not automatically remove zeros.
    
    See the Guava User Guide article on <a href=
    "https://github.com/google/guava/wiki/NewCollectionTypesExplained#multiset">`Multiset`</a>.

    Author(s)
    - Kevin Bourrillion

    Since
    - 2.0
    """

    def size(self) -> int:
        """
        Returns the total number of all occurrences of all elements in this multiset.
        
        **Note:** this method does not return the number of *distinct elements* in the
        multiset, which is given by `entrySet().size()`.
        """
        ...


    def count(self, element: "Object") -> int:
        """
        Returns the number of occurrences of an element in this multiset (the *count* of the
        element). Note that for an Object.equals-based multiset, this gives the same result as
        Collections.frequency (which would presumably perform more poorly).
        
        **Note:** the utility method Iterables.frequency generalizes this operation; it
        correctly delegates to this method when dealing with a multiset, but it can also accept any
        other iterable type.

        Arguments
        - element: the element to count occurrences of

        Returns
        - the number of occurrences of the element in this multiset; possibly zero but never
            negative
        """
        ...


    def add(self, element: "E", occurrences: int) -> int:
        """
        Adds a number of occurrences of an element to this multiset. Note that if `occurrences ==
        1`, this method has the identical effect to .add(Object). This method is functionally
        equivalent (except in the case of overflow) to the call `addAll(Collections.nCopies(element, occurrences))`, which would presumably perform much more
        poorly.

        Arguments
        - element: the element to add occurrences of; may be null only if explicitly allowed by the
            implementation
        - occurrences: the number of occurrences of the element to add. May be zero, in which case
            no change will be made.

        Returns
        - the count of the element before the operation; possibly zero

        Raises
        - IllegalArgumentException: if `occurrences` is negative, or if this operation would
            result in more than Integer.MAX_VALUE occurrences of the element
        - NullPointerException: if `element` is null and this implementation does not permit
            null elements. Note that if `occurrences` is zero, the implementation may opt to
            return normally.
        """
        ...


    def add(self, element: "E") -> bool:
        """
        Adds a single occurrence of the specified element to this multiset.
        
        This method refines Collection.add, which only *ensures* the presence of the
        element, to further specify that a successful call must always increment the count of the
        element, and the overall size of the collection, by one.
        
        To both add the element and obtain the previous count of that element, use .add(Object, int) add`(element, 1)` instead.

        Arguments
        - element: the element to add one occurrence of; may be null only if explicitly allowed by
            the implementation

        Returns
        - `True` always, since this call is required to modify the multiset, unlike other
            Collection types

        Raises
        - NullPointerException: if `element` is null and this implementation does not permit
            null elements
        - IllegalArgumentException: if Integer.MAX_VALUE occurrences of `element`
            are already contained in this multiset
        """
        ...


    def remove(self, element: "Object", occurrences: int) -> int:
        """
        Removes a number of occurrences of the specified element from this multiset. If the multiset
        contains fewer than this number of occurrences to begin with, all occurrences will be removed.
        Note that if `occurrences == 1`, this is functionally equivalent to the call `remove(element)`.

        Arguments
        - element: the element to conditionally remove occurrences of
        - occurrences: the number of occurrences of the element to remove. May be zero, in which
            case no change will be made.

        Returns
        - the count of the element before the operation; possibly zero

        Raises
        - IllegalArgumentException: if `occurrences` is negative
        """
        ...


    def remove(self, element: "Object") -> bool:
        """
        Removes a *single* occurrence of the specified element from this multiset, if present.
        
        This method refines Collection.remove to further specify that it **may not**
        throw an exception in response to `element` being null or of the wrong type.
        
        To both remove the element and obtain the previous count of that element, use .remove(Object, int) remove`(element, 1)` instead.

        Arguments
        - element: the element to remove one occurrence of

        Returns
        - `True` if an occurrence was found and removed
        """
        ...


    def setCount(self, element: "E", count: int) -> int:
        """
        Adds or removes the necessary occurrences of an element such that the element attains the
        desired count.

        Arguments
        - element: the element to add or remove occurrences of; may be null only if explicitly
            allowed by the implementation
        - count: the desired count of the element in this multiset

        Returns
        - the count of the element before the operation; possibly zero

        Raises
        - IllegalArgumentException: if `count` is negative
        - NullPointerException: if `element` is null and this implementation does not permit
            null elements. Note that if `count` is zero, the implementor may optionally return
            zero instead.
        """
        ...


    def setCount(self, element: "E", oldCount: int, newCount: int) -> bool:
        """
        Conditionally sets the count of an element to a new value, as described in .setCount(Object, int), provided that the element has the expected current count. If the
        current count is not `oldCount`, no change is made.

        Arguments
        - element: the element to conditionally set the count of; may be null only if explicitly
            allowed by the implementation
        - oldCount: the expected present count of the element in this multiset
        - newCount: the desired count of the element in this multiset

        Returns
        - `True` if the condition for modification was met. This implies that the multiset
            was indeed modified, unless `oldCount == newCount`.

        Raises
        - IllegalArgumentException: if `oldCount` or `newCount` is negative
        - NullPointerException: if `element` is null and the implementation does not permit
            null elements. Note that if `oldCount` and `newCount` are both zero, the
            implementor may optionally return `True` instead.
        """
        ...


    def elementSet(self) -> set["E"]:
        """
        Returns the set of distinct elements contained in this multiset. The element set is backed by
        the same data as the multiset, so any change to either is immediately reflected in the other.
        The order of the elements in the element set is unspecified.
        
        If the element set supports any removal operations, these necessarily cause **all**
        occurrences of the removed element(s) to be removed from the multiset. Implementations are not
        expected to support the add operations, although this is possible.
        
        A common use for the element set is to find the number of distinct elements in the multiset:
        `elementSet().size()`.

        Returns
        - a view of the set of distinct elements in this multiset
        """
        ...


    def entrySet(self) -> set["Entry"["E"]]:
        """
        Returns a view of the contents of this multiset, grouped into `Multiset.Entry` instances,
        each providing an element of the multiset and the count of that element. This set contains
        exactly one entry for each distinct element in the multiset (thus it always has the same size
        as the .elementSet). The order of the elements in the element set is unspecified.
        
        The entry set is backed by the same data as the multiset, so any change to either is
        immediately reflected in the other. However, multiset changes may or may not be reflected in
        any `Entry` instances already retrieved from the entry set (this is
        implementation-dependent). Furthermore, implementations are not required to support
        modifications to the entry set at all, and the `Entry` instances themselves don't even
        have methods for modification. See the specific implementation class for more details on how
        its entry set handles modifications.

        Returns
        - a set of entries representing the data of this multiset
        """
        ...


    def forEachEntry(self, action: "ObjIntConsumer"["E"]) -> None:
        """
        Runs the specified action for each distinct element in this multiset, and the number of
        occurrences of that element. For some `Multiset` implementations, this may be more
        efficient than iterating over the .entrySet() either explicitly or with `entrySet().forEach(action)`.

        Since
        - 21.0
        """
        ...


    def equals(self, object: "Object") -> bool:
        """
        Compares the specified object with this multiset for equality. Returns `True` if the
        given object is also a multiset and contains equal elements with equal counts, regardless of
        order.
        """
        ...


    def hashCode(self) -> int:
        """
        Returns the hash code for this multiset. This is defined as the sum of
        
        ````((element == null) ? 0 : element.hashCode()) ^ count(element)````
        
        over all distinct elements in the multiset. It follows that a multiset and its entry set
        always have the same hash code.
        """
        ...


    def toString(self) -> str:
        """
        
        
        It is recommended, though not mandatory, that this method return the result of invoking
        .toString on the .entrySet, yielding a result such as `[a x 3, c, d x 2,
        e]`.
        """
        ...


    def iterator(self) -> Iterator["E"]:
        """
        
        
        Elements that occur multiple times in the multiset will appear multiple times in this
        iterator, though not necessarily sequentially.
        """
        ...


    def contains(self, element: "Object") -> bool:
        """
        Determines whether this multiset contains the specified element.
        
        This method refines Collection.contains to further specify that it **may not**
        throw an exception in response to `element` being null or of the wrong type.

        Arguments
        - element: the element to check for

        Returns
        - `True` if this multiset contains at least one occurrence of the element
        """
        ...


    def containsAll(self, elements: Iterable[Any]) -> bool:
        """
        Returns `True` if this multiset contains at least one occurrence of each element in the
        specified collection.
        
        This method refines Collection.containsAll to further specify that it **may not**
        throw an exception in response to any of `elements` being null or of the wrong type.
        
        **Note:** this method does not take into account the occurrence count of an element in
        the two collections; it may still return `True` even if `elements` contains several
        occurrences of an element and this multiset contains only one. This is no different than any
        other collection type like List, but it may be unexpected to the user of a multiset.

        Arguments
        - elements: the collection of elements to be checked for containment in this multiset

        Returns
        - `True` if this multiset contains at least one occurrence of each element
            contained in `elements`

        Raises
        - NullPointerException: if `elements` is null
        """
        ...


    def removeAll(self, c: Iterable[Any]) -> bool:
        """
        
        
        **Note:** This method ignores how often any element might appear in `c`, and only
        cares whether or not an element appears at all. If you wish to remove one occurrence in this
        multiset for every occurrence in `c`, see Multisets.removeOccurrences(Multiset,
        Multiset).
        
        This method refines Collection.removeAll to further specify that it **may not**
        throw an exception in response to any of `elements` being null or of the wrong type.
        """
        ...


    def retainAll(self, c: Iterable[Any]) -> bool:
        """
        
        
        **Note:** This method ignores how often any element might appear in `c`, and only
        cares whether or not an element appears at all. If you wish to remove one occurrence in this
        multiset for every occurrence in `c`, see Multisets.retainOccurrences(Multiset,
        Multiset).
        
        This method refines Collection.retainAll to further specify that it **may not**
        throw an exception in response to any of `elements` being null or of the wrong type.

        See
        - Multisets.retainOccurrences(Multiset, Multiset)
        """
        ...


    def forEach(self, action: "Consumer"["E"]) -> None:
        """
        
        
        Elements that occur multiple times in the multiset will be passed to the `Consumer`
        correspondingly many times, though not necessarily sequentially.
        """
        ...


    def spliterator(self) -> "Spliterator"["E"]:
        ...



"""
Python module generated from Java source file org.bukkit.event.block.BlockExpEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block import Block
from org.bukkit.event import HandlerList
from org.bukkit.event.block import *
from typing import Any, Callable, Iterable, Tuple


class BlockExpEvent(BlockEvent):
    """
    An event that's called when a block yields experience.
    """

    def __init__(self, block: "Block", exp: int):
        ...


    def getExpToDrop(self) -> int:
        """
        Get the experience dropped by the block after the event has processed

        Returns
        - The experience to drop
        """
        ...


    def setExpToDrop(self, exp: int) -> None:
        """
        Set the amount of experience dropped by the block after the event has
        processed

        Arguments
        - exp: 1 or higher to drop experience, else nothing will drop
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file com.google.common.util.concurrent.GwtFluentFutureCatchingSpecialization

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.util.concurrent import *
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class GwtFluentFutureCatchingSpecialization(AbstractFuture):
    """
    Hidden superclass of FluentFuture that provides us a place to declare special GWT
    versions of the FluentFuture.catching(Class, com.google.common.base.Function)
    FluentFuture.catching family of methods. Those versions have slightly different signatures.
    """





"""
Python module generated from Java source file com.google.common.util.concurrent.CombinedFuture

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import ImmutableCollection
from com.google.common.util.concurrent import *
from com.google.errorprone.annotations.concurrent import LazyInit
from com.google.j2objc.annotations import WeakOuter
from java.util.concurrent import Callable
from java.util.concurrent import CancellationException
from java.util.concurrent import ExecutionException
from java.util.concurrent import Executor
from java.util.concurrent import RejectedExecutionException
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class CombinedFuture(AggregateFuture):
    """
    Aggregate future that computes its value by calling a callable.
    """





"""
Python module generated from Java source file java.io.ByteArrayInputStream

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import *
from java.util import Arrays
from java.util import Objects
from typing import Any, Callable, Iterable, Tuple


class ByteArrayInputStream(InputStream):
    """
    A `ByteArrayInputStream` contains
    an internal buffer that contains bytes that
    may be read from the stream. An internal
    counter keeps track of the next byte to
    be supplied by the `read` method.
    
    Closing a `ByteArrayInputStream` has no effect. The methods in
    this class can be called after the stream has been closed without
    generating an `IOException`.

    Author(s)
    - Arthur van Hoff

    See
    - java.io.StringBufferInputStream

    Since
    - 1.0
    """

    def __init__(self, buf: list[int]):
        """
        Creates a `ByteArrayInputStream`
        so that it  uses `buf` as its
        buffer array.
        The buffer array is not copied.
        The initial value of `pos`
        is `0` and the initial value
        of  `count` is the length of
        `buf`.

        Arguments
        - buf: the input buffer.
        """
        ...


    def __init__(self, buf: list[int], offset: int, length: int):
        """
        Creates `ByteArrayInputStream`
        that uses `buf` as its
        buffer array. The initial value of `pos`
        is `offset` and the initial value
        of `count` is the minimum of `offset+length`
        and `buf.length`.
        The buffer array is not copied. The buffer's mark is
        set to the specified offset.

        Arguments
        - buf: the input buffer.
        - offset: the offset in the buffer of the first byte to read.
        - length: the maximum number of bytes to read from the buffer.
        """
        ...


    def read(self) -> int:
        """
        Reads the next byte of data from this input stream. The value
        byte is returned as an `int` in the range
        `0` to `255`. If no byte is available
        because the end of the stream has been reached, the value
        `-1` is returned.
        
        This `read` method
        cannot block.

        Returns
        - the next byte of data, or `-1` if the end of the
                 stream has been reached.
        """
        ...


    def read(self, b: list[int], off: int, len: int) -> int:
        """
        Reads up to `len` bytes of data into an array of bytes from this
        input stream.  If `pos` equals `count`, then `-1` is
        returned to indicate end of file.  Otherwise, the  number `k` of
        bytes read is equal to the smaller of `len` and `count-pos`.
        If `k` is positive, then bytes `buf[pos]` through
        `buf[pos+k-1]` are copied into `b[off]` through
        `b[off+k-1]` in the manner performed by `System.arraycopy`.
        The value `k` is added into `pos` and `k` is returned.
        
        Unlike the InputStream.read(byte[],int,int) overridden method
        of `InputStream`, this method returns `-1` instead of zero
        if the end of the stream has been reached and `len == 0`.
        
        This `read` method cannot block.

        Arguments
        - b: the buffer into which the data is read.
        - off: the start offset in the destination array `b`
        - len: the maximum number of bytes read.

        Returns
        - the total number of bytes read into the buffer, or
                 `-1` if there is no more data because the end of
                 the stream has been reached.

        Raises
        - NullPointerException: If `b` is `null`.
        - IndexOutOfBoundsException: If `off` is negative,
        `len` is negative, or `len` is greater than
        `b.length - off`
        """
        ...


    def readAllBytes(self) -> list[int]:
        ...


    def readNBytes(self, b: list[int], off: int, len: int) -> int:
        ...


    def transferTo(self, out: "OutputStream") -> int:
        ...


    def skip(self, n: int) -> int:
        """
        Skips `n` bytes of input from this input stream. Fewer
        bytes might be skipped if the end of the input stream is reached.
        The actual number `k`
        of bytes to be skipped is equal to the smaller
        of `n` and  `count-pos`.
        The value `k` is added into `pos`
        and `k` is returned.

        Arguments
        - n: the number of bytes to be skipped.

        Returns
        - the actual number of bytes skipped.
        """
        ...


    def available(self) -> int:
        """
        Returns the number of remaining bytes that can be read (or skipped over)
        from this input stream.
        
        The value returned is `count - pos`,
        which is the number of bytes remaining to be read from the input buffer.

        Returns
        - the number of remaining bytes that can be read (or skipped
                 over) from this input stream without blocking.
        """
        ...


    def markSupported(self) -> bool:
        """
        Tests if this `InputStream` supports mark/reset. The
        `markSupported` method of `ByteArrayInputStream`
        always returns `True`.

        Since
        - 1.1
        """
        ...


    def mark(self, readAheadLimit: int) -> None:
        """
        Set the current marked position in the stream.
        ByteArrayInputStream objects are marked at position zero by
        default when constructed.  They may be marked at another
        position within the buffer by this method.
        
        If no mark has been set, then the value of the mark is the
        offset passed to the constructor (or 0 if the offset was not
        supplied).
        
         Note: The `readAheadLimit` for this class
         has no meaning.

        Since
        - 1.1
        """
        ...


    def reset(self) -> None:
        """
        Resets the buffer to the marked position.  The marked position
        is 0 unless another position was marked or an offset was specified
        in the constructor.
        """
        ...


    def close(self) -> None:
        """
        Closing a `ByteArrayInputStream` has no effect. The methods in
        this class can be called after the stream has been closed without
        generating an `IOException`.
        """
        ...



"""
Python module generated from Java source file org.bukkit.inventory.meta.BlockDataMeta

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Material
from org.bukkit.block.data import BlockData
from org.bukkit.inventory.meta import *
from typing import Any, Callable, Iterable, Tuple


class BlockDataMeta(ItemMeta):

    def hasBlockData(self) -> bool:
        """
        Returns whether the item has block data currently attached to it.

        Returns
        - whether block data is already attached
        """
        ...


    def getBlockData(self, material: "Material") -> "BlockData":
        """
        Returns the currently attached block data for this item or creates a new
        one if one doesn't exist.
        
        The state is a copy, it must be set back (or to another item) with
        .setBlockData(org.bukkit.block.data.BlockData)

        Arguments
        - material: the material we wish to get this data in the context of

        Returns
        - the attached data or new data
        """
        ...


    def setBlockData(self, blockData: "BlockData") -> None:
        """
        Attaches a copy of the passed block data to the item.

        Arguments
        - blockData: the block data to attach to the block.

        Raises
        - IllegalArgumentException: if the blockData is null or invalid for
        this item.
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.entity.EntityExhaustionEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.entity import HumanEntity
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.entity import *
from typing import Any, Callable, Iterable, Tuple


class EntityExhaustionEvent(EntityEvent, Cancellable):
    """
    Called when a human entity experiences exhaustion.
    
    An exhaustion level greater than 4.0 causes a decrease in saturation by 1.
    """

    def __init__(self, who: "HumanEntity", exhaustionReason: "ExhaustionReason", exhaustion: float):
        ...


    def getExhaustionReason(self) -> "ExhaustionReason":
        """
        Gets the ExhaustionReason for this event

        Returns
        - the exhaustion reason
        """
        ...


    def getExhaustion(self) -> float:
        """
        Get the amount of exhaustion to add to the player's current exhaustion.

        Returns
        - amount of exhaustion
        """
        ...


    def setExhaustion(self, exhaustion: float) -> None:
        """
        Set the exhaustion to apply to the player.
        
        The maximum exhaustion that a player can have is 40. No error will be
        thrown if this limit is hit. This value may be negative, but there is
        unknown behavior for when exhaustion is below 0.

        Arguments
        - exhaustion: new exhaustion to add
        """
        ...


    def getEntity(self) -> "HumanEntity":
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...


    class ExhaustionReason(Enum):
        """
        The reason for why a PlayerExhaustionEvent takes place
        """

        BLOCK_MINED = 0
        """
        Player mines a block
        """
        HUNGER_EFFECT = 1
        """
        Player has the hunger potion effect
        """
        DAMAGED = 2
        """
        Player takes damage
        """
        ATTACK = 3
        """
        Player attacks another entity
        """
        JUMP_SPRINT = 4
        """
        Player is sprint jumping
        """
        JUMP = 5
        """
        Player jumps
        """
        SWIM = 6
        """
        Player swims one centimeter
        """
        WALK_UNDERWATER = 7
        """
        Player walks underwater one centimeter
        """
        WALK_ON_WATER = 8
        """
        Player moves on the surface of water one centimeter
        """
        SPRINT = 9
        """
        Player sprints one centimeter
        """
        CROUCH = 10
        """
        Player crouches one centimeter (does not effect exhaustion, but fires
        nonetheless)
        """
        WALK = 11
        """
        Player walks one centimeter (does not effect exhaustion, but fires
        nonetheless)
        """
        REGEN = 12
        """
        Player regenerated health
        """
        UNKNOWN = 13
        """
        Unknown exhaustion reason
        """



"""
Python module generated from Java source file org.bukkit.TreeSpecies

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.collect import Maps
from enum import Enum
from org.bukkit import *
from typing import Any, Callable, Iterable, Tuple


class TreeSpecies(Enum):
    """
    Represents the different species of trees regardless of size.

    Deprecated
    - Deprecated, see usage methods for replacement(s)
    """

    GENERIC = (0x0)
    """
    Represents the common tree species.
    """
    REDWOOD = (0x1)
    """
    Represents the darker barked/leaved tree species.
    """
    BIRCH = (0x2)
    """
    Represents birches.
    """
    JUNGLE = (0x3)
    """
    Represents jungle trees.
    """
    ACACIA = (0x4)
    """
    Represents acacia trees.
    """
    DARK_OAK = (0x5)
    """
    Represents dark oak trees.
    """


    def getData(self) -> int:
        """
        Gets the associated data value representing this species

        Returns
        - A byte containing the data value of this tree species

        Deprecated
        - Magic value
        """
        ...


    @staticmethod
    def getByData(data: int) -> "TreeSpecies":
        """
        Gets the TreeSpecies with the given data value

        Arguments
        - data: Data value to fetch

        Returns
        - The TreeSpecies representing the given value, or null
            if it doesn't exist

        Deprecated
        - Magic value
        """
        ...



"""
Python module generated from Java source file org.bukkit.GameRule

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from org.bukkit import *
from typing import Any, Callable, Iterable, Tuple


class GameRule:
    """
    GameRules dictate certain behavior within Minecraft itself
    
    For more information please visit the
    <a href="https://minecraft.wiki/w/Commands/gamerule">Minecraft
    Wiki</a>
    
    Type `<T>`: type of rule (Boolean or Integer)
    """

    ANNOUNCE_ADVANCEMENTS = GameRule<>("announceAdvancements", Boolean.class)
    """
    Toggles the announcing of advancements.
    """
    COMMAND_BLOCK_OUTPUT = GameRule<>("commandBlockOutput", Boolean.class)
    """
    Whether command blocks should notify admins when they perform commands.
    """
    DISABLE_PLAYER_MOVEMENT_CHECK = GameRule<>("disablePlayerMovementCheck", Boolean.class)
    """
    Whether the server should skip checking player speed.
    """
    DISABLE_ELYTRA_MOVEMENT_CHECK = GameRule<>("disableElytraMovementCheck", Boolean.class)
    """
    Whether the server should skip checking player speed when the player is
    wearing elytra.
    """
    DO_DAYLIGHT_CYCLE = GameRule<>("doDaylightCycle", Boolean.class)
    """
    Whether time progresses from the current moment.
    """
    DO_ENTITY_DROPS = GameRule<>("doEntityDrops", Boolean.class)
    """
    Whether entities that are not mobs should have drops.
    """
    DO_FIRE_TICK = GameRule<>("doFireTick", Boolean.class)
    """
    Whether fire should spread and naturally extinguish.
    """
    DO_LIMITED_CRAFTING = GameRule<>("doLimitedCrafting", Boolean.class)
    """
    Whether players should only be able to craft recipes they've unlocked
    first.
    """
    DO_MOB_LOOT = GameRule<>("doMobLoot", Boolean.class)
    """
    Whether mobs should drop items.
    """
    PROJECTILES_CAN_BREAK_BLOCKS = GameRule<>("projectilesCanBreakBlocks", Boolean.class)
    """
    Whether projectiles can break blocks.
    """
    DO_MOB_SPAWNING = GameRule<>("doMobSpawning", Boolean.class)
    """
    Whether mobs should naturally spawn.
    """
    DO_TILE_DROPS = GameRule<>("doTileDrops", Boolean.class)
    """
    Whether blocks should have drops.
    """
    DO_WEATHER_CYCLE = GameRule<>("doWeatherCycle", Boolean.class)
    """
    Whether the weather will change from the current moment.
    """
    KEEP_INVENTORY = GameRule<>("keepInventory", Boolean.class)
    """
    Whether the player should keep items in their inventory after death.
    """
    LOG_ADMIN_COMMANDS = GameRule<>("logAdminCommands", Boolean.class)
    """
    Whether to log admin commands to server log.
    """
    MOB_GRIEFING = GameRule<>("mobGriefing", Boolean.class)
    """
    Whether mobs can pick up items or change blocks.
    """
    NATURAL_REGENERATION = GameRule<>("naturalRegeneration", Boolean.class)
    """
    Whether players can regenerate health naturally through their hunger bar.
    """
    REDUCED_DEBUG_INFO = GameRule<>("reducedDebugInfo", Boolean.class)
    """
    Whether the debug screen shows all or reduced information.
    """
    SEND_COMMAND_FEEDBACK = GameRule<>("sendCommandFeedback", Boolean.class)
    """
    Whether the feedback from commands executed by a player should show up in
    chat. Also affects the default behavior of whether command blocks store
    their output text.
    """
    SHOW_DEATH_MESSAGES = GameRule<>("showDeathMessages", Boolean.class)
    """
    Whether a message appears in chat when a player dies.
    """
    SPECTATORS_GENERATE_CHUNKS = GameRule<>("spectatorsGenerateChunks", Boolean.class)
    """
    Whether players in spectator mode can generate chunks.
    """
    DISABLE_RAIDS = GameRule<>("disableRaids", Boolean.class)
    """
    Whether pillager raids are enabled or not.
    """
    DO_INSOMNIA = GameRule<>("doInsomnia", Boolean.class)
    """
    Whether phantoms will appear without sleeping or not.
    """
    DO_IMMEDIATE_RESPAWN = GameRule<>("doImmediateRespawn", Boolean.class)
    """
    Whether clients will respawn immediately after death or not.
    """
    DROWNING_DAMAGE = GameRule<>("drowningDamage", Boolean.class)
    """
    Whether drowning damage is enabled or not.
    """
    FALL_DAMAGE = GameRule<>("fallDamage", Boolean.class)
    """
    Whether fall damage is enabled or not.
    """
    FIRE_DAMAGE = GameRule<>("fireDamage", Boolean.class)
    """
    Whether fire damage is enabled or not.
    """
    FREEZE_DAMAGE = GameRule<>("freezeDamage", Boolean.class)
    """
    Whether freeze damage is enabled or not.
    """
    DO_PATROL_SPAWNING = GameRule<>("doPatrolSpawning", Boolean.class)
    """
    Whether patrols should naturally spawn.
    """
    DO_TRADER_SPAWNING = GameRule<>("doTraderSpawning", Boolean.class)
    """
    Whether traders should naturally spawn.
    """
    DO_WARDEN_SPAWNING = GameRule<>("doWardenSpawning", Boolean.class)
    """
    Whether wardens should naturally spawn.
    """
    FORGIVE_DEAD_PLAYERS = GameRule<>("forgiveDeadPlayers", Boolean.class)
    """
    Whether mobs should cease being angry at a player once they die.
    """
    UNIVERSAL_ANGER = GameRule<>("universalAnger", Boolean.class)
    """
    Whether mobs will target all player entities once angered.
    """
    BLOCK_EXPLOSION_DROP_DECAY = GameRule<>("blockExplosionDropDecay", Boolean.class)
    """
    Whether block explosions will destroy dropped items.
    """
    MOB_EXPLOSION_DROP_DECAY = GameRule<>("mobExplosionDropDecay", Boolean.class)
    """
    * Whether mob explosions will destroy dropped items.
    """
    TNT_EXPLOSION_DROP_DECAY = GameRule<>("tntExplosionDropDecay", Boolean.class)
    """
    Whether tnt explosions will destroy dropped items.
    """
    WATER_SOURCE_CONVERSION = GameRule<>("waterSourceConversion", Boolean.class)
    """
    Whether water blocks can convert into water source blocks.
    """
    LAVA_SOURCE_CONVERSION = GameRule<>("lavaSourceConversion", Boolean.class)
    """
    Whether lava blocks can convert into lava source blocks.
    """
    GLOBAL_SOUND_EVENTS = GameRule<>("globalSoundEvents", Boolean.class)
    """
    Whether global level events such as ender dragon, wither, and completed
    end portal effects will propagate across the entire server.
    """
    DO_VINES_SPREAD = GameRule<>("doVinesSpread", Boolean.class)
    """
    Whether vines will spread.
    """
    ENDER_PEARLS_VANISH_ON_DEATH = GameRule<>("enderPearlsVanishOnDeath", Boolean.class)
    """
    Whether ender pearls will vanish on player death.
    """
    RANDOM_TICK_SPEED = GameRule<>("randomTickSpeed", Integer.class)
    """
    How often a random block tick occurs (such as plant growth, leaf decay,
    etc.) per chunk section per game tick. 0 will disable random ticks,
    higher numbers will increase random ticks.
    """
    SPAWN_RADIUS = GameRule<>("spawnRadius", Integer.class)
    """
    The number of blocks outward from the world spawn coordinates that a
    player will spawn in when first joining a server or when dying without a
    spawnpoint.
    """
    MAX_ENTITY_CRAMMING = GameRule<>("maxEntityCramming", Integer.class)
    """
    The maximum number of other pushable entities a mob or player can push,
    before taking suffocation damage.
    
    Setting to 0 disables this rule.
    """
    MAX_COMMAND_CHAIN_LENGTH = GameRule<>("maxCommandChainLength", Integer.class)
    """
    Determines the number at which the chain of command blocks act as a
    "chain."
    
    This is the maximum amount of command blocks that can be activated in a
    single tick from a single chain.
    """
    MAX_COMMAND_FORK_COUNT = GameRule<>("maxCommandForkCount", Integer.class)
    """
    Determines the number of different commands/functions which execute
    commands can fork into.
    """
    COMMAND_MODIFICATION_BLOCK_LIMIT = GameRule<>("commandModificationBlockLimit", Integer.class)
    """
    Determines the maximum number of blocks which a command can modify.
    """
    PLAYERS_SLEEPING_PERCENTAGE = GameRule<>("playersSleepingPercentage", Integer.class)
    """
    The percentage of online players which must be sleeping for the night to
    advance.
    """
    SNOW_ACCUMULATION_HEIGHT = GameRule<>("snowAccumulationHeight", Integer.class)
    PLAYERS_NETHER_PORTAL_DEFAULT_DELAY = GameRule<>("playersNetherPortalDefaultDelay", Integer.class)
    """
    The amount of time a player must stand in a nether portal before the
    portal activates.
    """
    PLAYERS_NETHER_PORTAL_CREATIVE_DELAY = GameRule<>("playersNetherPortalCreativeDelay", Integer.class)
    """
    The amount of time a player in creative mode must stand in a nether
    portal before the portal activates.
    """
    MINECART_MAX_SPEED = GameRule<>("minecartMaxSpeed", Integer.class)
    """
    The maximum speed of minecarts (when the new movement algorithm is
    enabled).
    """
    SPAWN_CHUNK_RADIUS = GameRule<>("spawnChunkRadius", Integer.class)
    """
    The number of chunks around spawn which will be kept loaded at all times.
    """


    def getName(self) -> str:
        """
        Get the name of this GameRule.

        Returns
        - the name of this GameRule
        """
        ...


    def getType(self) -> type["T"]:
        """
        Get the type of this rule.

        Returns
        - the rule type; Integer or Boolean
        """
        ...


    def equals(self, obj: "Object") -> bool:
        ...


    def toString(self) -> str:
        ...


    @staticmethod
    def getByName(rule: str) -> "GameRule"[Any]:
        """
        Get a GameRule by its name.

        Arguments
        - rule: the name of the GameRule

        Returns
        - the GameRule or null if no GameRule matches the given
        name
        """
        ...


    @staticmethod
    def values() -> list["GameRule"[Any]]:
        """
        Get an immutable collection of GameRules.

        Returns
        - an immutable collection containing all registered GameRules.
        """
        ...



"""
Python module generated from Java source file com.google.common.util.concurrent.CycleDetectingLockFactory

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.annotations import VisibleForTesting
from com.google.common.base import MoreObjects
from com.google.common.base import Preconditions
from com.google.common.collect import ImmutableSet
from com.google.common.collect import Lists
from com.google.common.collect import MapMaker
from com.google.common.collect import Maps
from com.google.common.collect import Sets
from com.google.common.util.concurrent import *
from com.google.j2objc.annotations import Weak
from enum import Enum
from java.util import Arrays
from java.util import Collections
from java.util import EnumMap
from java.util.concurrent import ConcurrentMap
from java.util.concurrent import TimeUnit
from java.util.concurrent.locks import ReentrantLock
from java.util.concurrent.locks import ReentrantReadWriteLock
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class CycleDetectingLockFactory:
    """
    The `CycleDetectingLockFactory` creates ReentrantLock instances and ReentrantReadWriteLock instances that detect potential deadlock by checking for cycles in lock
    acquisition order.
    
    Potential deadlocks detected when calling the `lock()`, `lockInterruptibly()`, or
    `tryLock()` methods will result in the execution of the Policy specified when
    creating the factory. The currently available policies are:
    
    
      - DISABLED
      - WARN
      - THROW
    
    
    The locks created by a factory instance will detect lock acquisition cycles with locks created
    by other `CycleDetectingLockFactory` instances (except those with `Policy.DISABLED`).
    A lock's behavior when a cycle is detected, however, is defined by the `Policy` of the
    factory that created it. This allows detection of cycles across components while delegating
    control over lock behavior to individual components.
    
    Applications are encouraged to use a `CycleDetectingLockFactory` to create any locks for
    which external/unmanaged code is executed while the lock is held. (See caveats under
    <strong>Performance</strong>).
    
    <strong>Cycle Detection</strong>
    
    Deadlocks can arise when locks are acquired in an order that forms a cycle. In a simple
    example involving two locks and two threads, deadlock occurs when one thread acquires Lock A, and
    then Lock B, while another thread acquires Lock B, and then Lock A:
    
    ```
    Thread1: acquire(LockA) --X acquire(LockB)
    Thread2: acquire(LockB) --X acquire(LockA)
    ```
    
    Neither thread will progress because each is waiting for the other. In more complex
    applications, cycles can arise from interactions among more than 2 locks:
    
    ```
    Thread1: acquire(LockA) --X acquire(LockB)
    Thread2: acquire(LockB) --X acquire(LockC)
    ...
    ThreadN: acquire(LockN) --X acquire(LockA)
    ```
    
    The implementation detects cycles by constructing a directed graph in which each lock
    represents a node and each edge represents an acquisition ordering between two locks.
    
    
      - Each lock adds (and removes) itself to/from a ThreadLocal Set of acquired locks when the
          Thread acquires its first hold (and releases its last remaining hold).
      - Before the lock is acquired, the lock is checked against the current set of acquired
          locks---to each of the acquired locks, an edge from the soon-to-be-acquired lock is either
          verified or created.
      - If a new edge needs to be created, the outgoing edges of the acquired locks are traversed
          to check for a cycle that reaches the lock to be acquired. If no cycle is detected, a new
          "safe" edge is created.
      - If a cycle is detected, an "unsafe" (cyclic) edge is created to represent a potential
          deadlock situation, and the appropriate Policy is executed.
    
    
    Note that detection of potential deadlock does not necessarily indicate that deadlock will
    happen, as it is possible that higher level application logic prevents the cyclic lock
    acquisition from occurring. One example of a False positive is:
    
    ```
    LockA -&gt; LockB -&gt; LockC
    LockA -&gt; LockC -&gt; LockB
    ```
    
    <strong>ReadWriteLocks</strong>
    
    While `ReadWriteLock` instances have different properties and can form cycles without
    potential deadlock, this class treats `ReadWriteLock` instances as equivalent to
    traditional exclusive locks. Although this increases the False positives that the locks detect
    (i.e. cycles that will not actually result in deadlock), it simplifies the algorithm and
    implementation considerably. The assumption is that a user of this factory wishes to eliminate
    any cyclic acquisition ordering.
    
    <strong>Explicit Lock Acquisition Ordering</strong>
    
    The CycleDetectingLockFactory.WithExplicitOrdering class can be used to enforce an
    application-specific ordering in addition to performing general cycle detection.
    
    <strong>Garbage Collection</strong>
    
    In order to allow proper garbage collection of unused locks, the edges of the lock graph are
    weak references.
    
    <strong>Performance</strong>
    
    The extra bookkeeping done by cycle detecting locks comes at some cost to performance.
    Benchmarks (as of December 2011) show that:
    
    
      - for an unnested `lock()` and `unlock()`, a cycle detecting lock takes 38ns as
          opposed to the 24ns taken by a plain lock.
      - for nested locking, the cost increases with the depth of the nesting:
          
            - 2 levels: average of 64ns per lock()/unlock()
            - 3 levels: average of 77ns per lock()/unlock()
            - 4 levels: average of 99ns per lock()/unlock()
            - 5 levels: average of 103ns per lock()/unlock()
            - 10 levels: average of 184ns per lock()/unlock()
            - 20 levels: average of 393ns per lock()/unlock()
          
    
    
    As such, the CycleDetectingLockFactory may not be suitable for performance-critical
    applications which involve tightly-looped or deeply-nested locking algorithms.

    Author(s)
    - Darick Tong

    Since
    - 13.0
    """

    @staticmethod
    def newInstance(policy: "Policy") -> "CycleDetectingLockFactory":
        """
        Creates a new factory with the specified policy.
        """
        ...


    def newReentrantLock(self, lockName: str) -> "ReentrantLock":
        """
        Equivalent to `newReentrantLock(lockName, False)`.
        """
        ...


    def newReentrantLock(self, lockName: str, fair: bool) -> "ReentrantLock":
        """
        Creates a ReentrantLock with the given fairness policy. The `lockName` is used in
        the warning or exception output to help identify the locks involved in the detected deadlock.
        """
        ...


    def newReentrantReadWriteLock(self, lockName: str) -> "ReentrantReadWriteLock":
        """
        Equivalent to `newReentrantReadWriteLock(lockName, False)`.
        """
        ...


    def newReentrantReadWriteLock(self, lockName: str, fair: bool) -> "ReentrantReadWriteLock":
        """
        Creates a ReentrantReadWriteLock with the given fairness policy. The `lockName`
        is used in the warning or exception output to help identify the locks involved in the detected
        deadlock.
        """
        ...


    @staticmethod
    def newInstanceWithExplicitOrdering(enumClass: type["E"], policy: "Policy") -> "WithExplicitOrdering"["E"]:
        """
        Creates a `CycleDetectingLockFactory.WithExplicitOrdering<E>`.
        """
        ...


    class Policy:
        """
        Encapsulates the action to be taken when a potential deadlock is encountered. Clients can use
        one of the predefined Policies or specify a custom implementation. Implementations must
        be thread-safe.

        Since
        - 13.0
        """

        def handlePotentialDeadlock(self, exception: "PotentialDeadlockException") -> None:
            """
            Called when a potential deadlock is encountered. Implementations can throw the given `exception` and/or execute other desired logic.
            
            Note that the method will be called even upon an invocation of `tryLock()`. Although
            `tryLock()` technically recovers from deadlock by eventually timing out, this behavior
            is chosen based on the assumption that it is the application's wish to prohibit any cyclical
            lock acquisitions.
            """
            ...


    class WithExplicitOrdering(CycleDetectingLockFactory):
        """
        A `CycleDetectingLockFactory.WithExplicitOrdering` provides the additional enforcement of
        an application-specified ordering of lock acquisitions. The application defines the allowed
        ordering with an `Enum` whose values each correspond to a lock type. The order in which
        the values are declared dictates the allowed order of lock acquisition. In other words, locks
        corresponding to smaller values of Enum.ordinal() should only be acquired before locks
        with larger ordinals. Example:
        
        ````enum MyLockOrder {
          FIRST, SECOND, THIRD;`
        
        CycleDetectingLockFactory.WithExplicitOrdering<MyLockOrder> factory =
          CycleDetectingLockFactory.newInstanceWithExplicitOrdering(Policies.THROW);
        
        Lock lock1 = factory.newReentrantLock(MyLockOrder.FIRST);
        Lock lock2 = factory.newReentrantLock(MyLockOrder.SECOND);
        Lock lock3 = factory.newReentrantLock(MyLockOrder.THIRD);
        
        lock1.lock();
        lock3.lock();
        lock2.lock();  // will throw an IllegalStateException
        }```
        
        As with all locks created by instances of `CycleDetectingLockFactory` explicitly
        ordered locks participate in general cycle detection with all other cycle detecting locks, and
        a lock's behavior when detecting a cyclic lock acquisition is defined by the `Policy` of
        the factory that created it.
        
        Note, however, that although multiple locks can be created for a given Enum value, whether
        it be through separate factory instances or through multiple calls to the same factory,
        attempting to acquire multiple locks with the same Enum value (within the same thread) will
        result in an IllegalStateException regardless of the factory's policy. For example:
        
        ````CycleDetectingLockFactory.WithExplicitOrdering<MyLockOrder> factory1 =
          CycleDetectingLockFactory.newInstanceWithExplicitOrdering(...);
        CycleDetectingLockFactory.WithExplicitOrdering<MyLockOrder> factory2 =
          CycleDetectingLockFactory.newInstanceWithExplicitOrdering(...);
        
        Lock lockA = factory1.newReentrantLock(MyLockOrder.FIRST);
        Lock lockB = factory1.newReentrantLock(MyLockOrder.FIRST);
        Lock lockC = factory2.newReentrantLock(MyLockOrder.FIRST);
        
        lockA.lock();
        
        lockB.lock();  // will throw an IllegalStateException
        lockC.lock();  // will throw an IllegalStateException
        
        lockA.lock();  // reentrant acquisition is okay````
        
        It is the responsibility of the application to ensure that multiple lock instances with the
        same rank are never acquired in the same thread.
        
        Type `<E>`: The Enum type representing the explicit lock ordering.

        Since
        - 13.0
        """

        def newReentrantLock(self, rank: "E") -> "ReentrantLock":
            """
            Equivalent to `newReentrantLock(rank, False)`.
            """
            ...


        def newReentrantLock(self, rank: "E", fair: bool) -> "ReentrantLock":
            """
            Creates a ReentrantLock with the given fairness policy and rank. The values returned
            by Enum.getDeclaringClass() and Enum.name() are used to describe the lock in
            warning or exception output.

            Raises
            - IllegalStateException: If the factory has already created a `Lock` with the
                specified rank.
            """
            ...


        def newReentrantReadWriteLock(self, rank: "E") -> "ReentrantReadWriteLock":
            """
            Equivalent to `newReentrantReadWriteLock(rank, False)`.
            """
            ...


        def newReentrantReadWriteLock(self, rank: "E", fair: bool) -> "ReentrantReadWriteLock":
            """
            Creates a ReentrantReadWriteLock with the given fairness policy and rank. The values
            returned by Enum.getDeclaringClass() and Enum.name() are used to describe the
            lock in warning or exception output.

            Raises
            - IllegalStateException: If the factory has already created a `Lock` with the
                specified rank.
            """
            ...


    class PotentialDeadlockException(ExampleStackTrace):
        """
        Represents a detected cycle in lock acquisition ordering. The exception includes a causal chain
        of `ExampleStackTrace` instances to illustrate the cycle, e.g.
        
        ```
        com....PotentialDeadlockException: Potential Deadlock from LockC -&gt; ReadWriteA
          at ...
          at ...
        Caused by: com...ExampleStackTrace: LockB -&gt; LockC
          at ...
          at ...
        Caused by: com...ExampleStackTrace: ReadWriteA -&gt; LockB
          at ...
          at ...
        ```
        
        Instances are logged for the `Policies.WARN`, and thrown for `Policies.THROW`.

        Since
        - 13.0
        """

        def getConflictingStackTrace(self) -> "ExampleStackTrace":
            ...


        def getMessage(self) -> str:
            """
            Appends the chain of messages from the `conflictingStackTrace` to the original `message`.
            """
            ...


    class Policies(Enum):
        """
        Pre-defined Policy implementations.

        Since
        - 13.0
        """

        THROW = 0
        """
        When potential deadlock is detected, this policy results in the throwing of the `PotentialDeadlockException` indicating the potential deadlock, which includes stack traces
        illustrating the cycle in lock acquisition order.
        """
        WARN = 1
        """
        When potential deadlock is detected, this policy results in the logging of a Level.SEVERE message indicating the potential deadlock, which includes stack traces
        illustrating the cycle in lock acquisition order.
        """
        DISABLED = 2
        """
        Disables cycle detection. This option causes the factory to return unmodified lock
        implementations provided by the JDK, and is provided to allow applications to easily
        parameterize when cycle detection is enabled.
        
        Note that locks created by a factory with this policy will *not* participate the
        cycle detection performed by locks created by other factories.
        """



"""
Python module generated from Java source file java.util.concurrent.ScheduledThreadPoolExecutor

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import AbstractQueue
from java.util import Arrays
from java.util import Iterator
from java.util import NoSuchElementException
from java.util import Objects
from java.util.concurrent import *
from java.util.concurrent.atomic import AtomicLong
from java.util.concurrent.locks import Condition
from java.util.concurrent.locks import ReentrantLock
from typing import Any, Callable, Iterable, Tuple


class ScheduledThreadPoolExecutor(ThreadPoolExecutor, ScheduledExecutorService):
    """
    A ThreadPoolExecutor that can additionally schedule
    commands to run after a given delay, or to execute periodically.
    This class is preferable to java.util.Timer when multiple
    worker threads are needed, or when the additional flexibility or
    capabilities of ThreadPoolExecutor (which this class
    extends) are required.
    
    Delayed tasks execute no sooner than they are enabled, but
    without any real-time guarantees about when, after they are
    enabled, they will commence. Tasks scheduled for exactly the same
    execution time are enabled in first-in-first-out (FIFO) order of
    submission.
    
    When a submitted task is cancelled before it is run, execution
    is suppressed.  By default, such a cancelled task is not
    automatically removed from the work queue until its delay elapses.
    While this enables further inspection and monitoring, it may also
    cause unbounded retention of cancelled tasks.  To avoid this, use
    .setRemoveOnCancelPolicy to cause tasks to be immediately
    removed from the work queue at time of cancellation.
    
    Successive executions of a periodic task scheduled via
    .scheduleAtFixedRate scheduleAtFixedRate or
    .scheduleWithFixedDelay scheduleWithFixedDelay
    do not overlap. While different executions may be performed by
    different threads, the effects of prior executions
    <a href="package-summary.html#MemoryVisibility">*happen-before*</a>
    those of subsequent ones.
    
    While this class inherits from ThreadPoolExecutor, a few
    of the inherited tuning methods are not useful for it. In
    particular, because it acts as a fixed-sized pool using
    `corePoolSize` threads and an unbounded queue, adjustments
    to `maximumPoolSize` have no useful effect. Additionally, it
    is almost never a good idea to set `corePoolSize` to zero or
    use `allowCoreThreadTimeOut` because this may leave the pool
    without threads to handle tasks once they become eligible to run.
    
    As with `ThreadPoolExecutor`, if not otherwise specified,
    this class uses Executors.defaultThreadFactory as the
    default thread factory, and ThreadPoolExecutor.AbortPolicy
    as the default rejected execution handler.
    
    **Extension notes:** This class overrides the
    ThreadPoolExecutor.execute(Runnable) execute and
    AbstractExecutorService.submit(Runnable) submit
    methods to generate internal ScheduledFuture objects to
    control per-task delays and scheduling.  To preserve
    functionality, any further overrides of these methods in
    subclasses must invoke superclass versions, which effectively
    disables additional task customization.  However, this class
    provides alternative protected extension method
    `decorateTask` (one version each for `Runnable` and
    `Callable`) that can be used to customize the concrete task
    types used to execute commands entered via `execute`,
    `submit`, `schedule`, `scheduleAtFixedRate`,
    and `scheduleWithFixedDelay`.  By default, a
    `ScheduledThreadPoolExecutor` uses a task type extending
    FutureTask. However, this may be modified or replaced using
    subclasses of the form:
    
    ``` `public class CustomScheduledExecutor extends ScheduledThreadPoolExecutor {
    
      static class CustomTask<V> implements RunnableScheduledFuture<V> { ...`
    
      protected <V> RunnableScheduledFuture<V> decorateTask(
                   Runnable r, RunnableScheduledFuture<V> task) {
          return new CustomTask<V>(r, task);
      }
    
      protected <V> RunnableScheduledFuture<V> decorateTask(
                   Callable<V> c, RunnableScheduledFuture<V> task) {
          return new CustomTask<V>(c, task);
      }
      // ... add constructors, etc.
    }}```

    Author(s)
    - Doug Lea

    Since
    - 1.5
    """

    def __init__(self, corePoolSize: int):
        """
        Creates a new `ScheduledThreadPoolExecutor` with the
        given core pool size.

        Arguments
        - corePoolSize: the number of threads to keep in the pool, even
               if they are idle, unless `allowCoreThreadTimeOut` is set

        Raises
        - IllegalArgumentException: if `corePoolSize < 0`
        """
        ...


    def __init__(self, corePoolSize: int, threadFactory: "ThreadFactory"):
        """
        Creates a new `ScheduledThreadPoolExecutor` with the
        given initial parameters.

        Arguments
        - corePoolSize: the number of threads to keep in the pool, even
               if they are idle, unless `allowCoreThreadTimeOut` is set
        - threadFactory: the factory to use when the executor
               creates a new thread

        Raises
        - IllegalArgumentException: if `corePoolSize < 0`
        - NullPointerException: if `threadFactory` is null
        """
        ...


    def __init__(self, corePoolSize: int, handler: "RejectedExecutionHandler"):
        """
        Creates a new `ScheduledThreadPoolExecutor` with the
        given initial parameters.

        Arguments
        - corePoolSize: the number of threads to keep in the pool, even
               if they are idle, unless `allowCoreThreadTimeOut` is set
        - handler: the handler to use when execution is blocked
               because the thread bounds and queue capacities are reached

        Raises
        - IllegalArgumentException: if `corePoolSize < 0`
        - NullPointerException: if `handler` is null
        """
        ...


    def __init__(self, corePoolSize: int, threadFactory: "ThreadFactory", handler: "RejectedExecutionHandler"):
        """
        Creates a new `ScheduledThreadPoolExecutor` with the
        given initial parameters.

        Arguments
        - corePoolSize: the number of threads to keep in the pool, even
               if they are idle, unless `allowCoreThreadTimeOut` is set
        - threadFactory: the factory to use when the executor
               creates a new thread
        - handler: the handler to use when execution is blocked
               because the thread bounds and queue capacities are reached

        Raises
        - IllegalArgumentException: if `corePoolSize < 0`
        - NullPointerException: if `threadFactory` or
                `handler` is null
        """
        ...


    def schedule(self, command: "Runnable", delay: int, unit: "TimeUnit") -> "ScheduledFuture"[Any]:
        """
        Raises
        - RejectedExecutionException: 
        - NullPointerException: 
        """
        ...


    def schedule(self, callable: "Callable"["V"], delay: int, unit: "TimeUnit") -> "ScheduledFuture"["V"]:
        """
        Raises
        - RejectedExecutionException: 
        - NullPointerException: 
        """
        ...


    def scheduleAtFixedRate(self, command: "Runnable", initialDelay: int, period: int, unit: "TimeUnit") -> "ScheduledFuture"[Any]:
        """
        Submits a periodic action that becomes enabled first after the
        given initial delay, and subsequently with the given period;
        that is, executions will commence after
        `initialDelay`, then `initialDelay + period`, then
        `initialDelay + 2 * period`, and so on.
        
        The sequence of task executions continues indefinitely until
        one of the following exceptional completions occur:
        
        - The task is Future.cancel explicitly cancelled
        via the returned future.
        - Method .shutdown is called and the .getContinueExistingPeriodicTasksAfterShutdownPolicy policy on
        whether to continue after shutdown is not set True, or method
        .shutdownNow is called; also resulting in task
        cancellation.
        - An execution of the task throws an exception.  In this case
        calling Future.get() get on the returned future will throw
        ExecutionException, holding the exception as its cause.
        
        Subsequent executions are suppressed.  Subsequent calls to
        Future.isDone isDone() on the returned future will
        return `True`.
        
        If any execution of this task takes longer than its period, then
        subsequent executions may start late, but will not concurrently
        execute.

        Raises
        - RejectedExecutionException: 
        - NullPointerException: 
        - IllegalArgumentException: 
        """
        ...


    def scheduleWithFixedDelay(self, command: "Runnable", initialDelay: int, delay: int, unit: "TimeUnit") -> "ScheduledFuture"[Any]:
        """
        Submits a periodic action that becomes enabled first after the
        given initial delay, and subsequently with the given delay
        between the termination of one execution and the commencement of
        the next.
        
        The sequence of task executions continues indefinitely until
        one of the following exceptional completions occur:
        
        - The task is Future.cancel explicitly cancelled
        via the returned future.
        - Method .shutdown is called and the .getContinueExistingPeriodicTasksAfterShutdownPolicy policy on
        whether to continue after shutdown is not set True, or method
        .shutdownNow is called; also resulting in task
        cancellation.
        - An execution of the task throws an exception.  In this case
        calling Future.get() get on the returned future will throw
        ExecutionException, holding the exception as its cause.
        
        Subsequent executions are suppressed.  Subsequent calls to
        Future.isDone isDone() on the returned future will
        return `True`.

        Raises
        - RejectedExecutionException: 
        - NullPointerException: 
        - IllegalArgumentException: 
        """
        ...


    def execute(self, command: "Runnable") -> None:
        """
        Executes `command` with zero required delay.
        This has effect equivalent to
        .schedule(Runnable,long,TimeUnit) schedule(command, 0, anyUnit).
        Note that inspections of the queue and of the list returned by
        `shutdownNow` will access the zero-delayed
        ScheduledFuture, not the `command` itself.
        
        A consequence of the use of `ScheduledFuture` objects is
        that ThreadPoolExecutor.afterExecute afterExecute is always
        called with a null second `Throwable` argument, even if the
        `command` terminated abruptly.  Instead, the `Throwable`
        thrown by such a task can be obtained via Future.get.

        Raises
        - RejectedExecutionException: at discretion of
                `RejectedExecutionHandler`, if the task
                cannot be accepted for execution because the
                executor has been shut down
        - NullPointerException: 
        """
        ...


    def submit(self, task: "Runnable") -> "Future"[Any]:
        """
        Raises
        - RejectedExecutionException: 
        - NullPointerException: 
        """
        ...


    def submit(self, task: "Runnable", result: "T") -> "Future"["T"]:
        """
        Raises
        - RejectedExecutionException: 
        - NullPointerException: 
        """
        ...


    def submit(self, task: "Callable"["T"]) -> "Future"["T"]:
        """
        Raises
        - RejectedExecutionException: 
        - NullPointerException: 
        """
        ...


    def setContinueExistingPeriodicTasksAfterShutdownPolicy(self, value: bool) -> None:
        """
        Sets the policy on whether to continue executing existing
        periodic tasks even when this executor has been `shutdown`.
        In this case, executions will continue until `shutdownNow`
        or the policy is set to `False` when already shutdown.
        This value is by default `False`.

        Arguments
        - value: if `True`, continue after shutdown, else don't

        See
        - .getContinueExistingPeriodicTasksAfterShutdownPolicy
        """
        ...


    def getContinueExistingPeriodicTasksAfterShutdownPolicy(self) -> bool:
        """
        Gets the policy on whether to continue executing existing
        periodic tasks even when this executor has been `shutdown`.
        In this case, executions will continue until `shutdownNow`
        or the policy is set to `False` when already shutdown.
        This value is by default `False`.

        Returns
        - `True` if will continue after shutdown

        See
        - .setContinueExistingPeriodicTasksAfterShutdownPolicy
        """
        ...


    def setExecuteExistingDelayedTasksAfterShutdownPolicy(self, value: bool) -> None:
        """
        Sets the policy on whether to execute existing delayed
        tasks even when this executor has been `shutdown`.
        In this case, these tasks will only terminate upon
        `shutdownNow`, or after setting the policy to
        `False` when already shutdown.
        This value is by default `True`.

        Arguments
        - value: if `True`, execute after shutdown, else don't

        See
        - .getExecuteExistingDelayedTasksAfterShutdownPolicy
        """
        ...


    def getExecuteExistingDelayedTasksAfterShutdownPolicy(self) -> bool:
        """
        Gets the policy on whether to execute existing delayed
        tasks even when this executor has been `shutdown`.
        In this case, these tasks will only terminate upon
        `shutdownNow`, or after setting the policy to
        `False` when already shutdown.
        This value is by default `True`.

        Returns
        - `True` if will execute after shutdown

        See
        - .setExecuteExistingDelayedTasksAfterShutdownPolicy
        """
        ...


    def setRemoveOnCancelPolicy(self, value: bool) -> None:
        """
        Sets the policy on whether cancelled tasks should be immediately
        removed from the work queue at time of cancellation.  This value is
        by default `False`.

        Arguments
        - value: if `True`, remove on cancellation, else don't

        See
        - .getRemoveOnCancelPolicy

        Since
        - 1.7
        """
        ...


    def getRemoveOnCancelPolicy(self) -> bool:
        """
        Gets the policy on whether cancelled tasks should be immediately
        removed from the work queue at time of cancellation.  This value is
        by default `False`.

        Returns
        - `True` if cancelled tasks are immediately removed
                from the queue

        See
        - .setRemoveOnCancelPolicy

        Since
        - 1.7
        """
        ...


    def shutdown(self) -> None:
        """
        Initiates an orderly shutdown in which previously submitted
        tasks are executed, but no new tasks will be accepted.
        Invocation has no additional effect if already shut down.
        
        This method does not wait for previously submitted tasks to
        complete execution.  Use .awaitTermination awaitTermination
        to do that.
        
        If the `ExecuteExistingDelayedTasksAfterShutdownPolicy`
        has been set `False`, existing delayed tasks whose delays
        have not yet elapsed are cancelled.  And unless the `ContinueExistingPeriodicTasksAfterShutdownPolicy` has been set
        `True`, future executions of existing periodic tasks will
        be cancelled.

        Raises
        - SecurityException: 
        """
        ...


    def shutdownNow(self) -> list["Runnable"]:
        """
        Attempts to stop all actively executing tasks, halts the
        processing of waiting tasks, and returns a list of the tasks
        that were awaiting execution. These tasks are drained (removed)
        from the task queue upon return from this method.
        
        This method does not wait for actively executing tasks to
        terminate.  Use .awaitTermination awaitTermination to
        do that.
        
        There are no guarantees beyond best-effort attempts to stop
        processing actively executing tasks.  This implementation
        interrupts tasks via Thread.interrupt; any task that
        fails to respond to interrupts may never terminate.

        Returns
        - list of tasks that never commenced execution.
                Each element of this list is a ScheduledFuture.
                For tasks submitted via one of the `schedule`
                methods, the element will be identical to the returned
                `ScheduledFuture`.  For tasks submitted using
                .execute execute, the element will be a
                zero-delay `ScheduledFuture`.

        Raises
        - SecurityException: 
        """
        ...


    def getQueue(self) -> "BlockingQueue"["Runnable"]:
        """
        Returns the task queue used by this executor.  Access to the
        task queue is intended primarily for debugging and monitoring.
        This queue may be in active use.  Retrieving the task queue
        does not prevent queued tasks from executing.
        
        Each element of this queue is a ScheduledFuture.
        For tasks submitted via one of the `schedule` methods, the
        element will be identical to the returned `ScheduledFuture`.
        For tasks submitted using .execute execute, the element
        will be a zero-delay `ScheduledFuture`.
        
        Iteration over this queue is *not* guaranteed to traverse
        tasks in the order in which they will execute.

        Returns
        - the task queue
        """
        ...



"""
Python module generated from Java source file org.bukkit.advancement.Advancement

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Keyed
from org.bukkit.advancement import *
from typing import Any, Callable, Iterable, Tuple


class Advancement(Keyed):
    """
    Represents an advancement that may be awarded to a player. This class is not
    reference safe as the underlying advancement may be reloaded.
    """

    def getCriteria(self) -> Iterable[str]:
        """
        Get all the criteria present in this advancement.

        Returns
        - a unmodifiable copy of all criteria
        """
        ...


    def getRequirements(self) -> "AdvancementRequirements":
        """
        Returns the requirements for this advancement.

        Returns
        - an AdvancementRequirements object.
        """
        ...


    def getDisplay(self) -> "AdvancementDisplay":
        """
        Returns the display information for this advancement.
        
        This includes it's name, description and other visible tags.

        Returns
        - a AdvancementDisplay object, or null if not set.
        """
        ...



"""
Python module generated from Java source file org.bukkit.generator.structure.Structure

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Keyed
from org.bukkit import NamespacedKey
from org.bukkit import Registry
from org.bukkit.generator.structure import *
from org.bukkit.registry import RegistryAware
from typing import Any, Callable, Iterable, Tuple


class Structure(Keyed, RegistryAware):
    """
    Represent a Structure from the world.
    
    Listed structures are present in the default server. Depending on the server
    there might be additional structures present (for example structures added by
    data packs), which can be received via Registry.STRUCTURE.
    """

    PILLAGER_OUTPOST = getStructure("pillager_outpost")
    MINESHAFT = getStructure("mineshaft")
    MINESHAFT_MESA = getStructure("mineshaft_mesa")
    MANSION = getStructure("mansion")
    JUNGLE_PYRAMID = getStructure("jungle_pyramid")
    DESERT_PYRAMID = getStructure("desert_pyramid")
    IGLOO = getStructure("igloo")
    SHIPWRECK = getStructure("shipwreck")
    SHIPWRECK_BEACHED = getStructure("shipwreck_beached")
    SWAMP_HUT = getStructure("swamp_hut")
    STRONGHOLD = getStructure("stronghold")
    MONUMENT = getStructure("monument")
    OCEAN_RUIN_COLD = getStructure("ocean_ruin_cold")
    OCEAN_RUIN_WARM = getStructure("ocean_ruin_warm")
    FORTRESS = getStructure("fortress")
    NETHER_FOSSIL = getStructure("nether_fossil")
    END_CITY = getStructure("end_city")
    BURIED_TREASURE = getStructure("buried_treasure")
    BASTION_REMNANT = getStructure("bastion_remnant")
    VILLAGE_PLAINS = getStructure("village_plains")
    VILLAGE_DESERT = getStructure("village_desert")
    VILLAGE_SAVANNA = getStructure("village_savanna")
    VILLAGE_SNOWY = getStructure("village_snowy")
    VILLAGE_TAIGA = getStructure("village_taiga")
    RUINED_PORTAL = getStructure("ruined_portal")
    RUINED_PORTAL_DESERT = getStructure("ruined_portal_desert")
    RUINED_PORTAL_JUNGLE = getStructure("ruined_portal_jungle")
    RUINED_PORTAL_SWAMP = getStructure("ruined_portal_swamp")
    RUINED_PORTAL_MOUNTAIN = getStructure("ruined_portal_mountain")
    RUINED_PORTAL_OCEAN = getStructure("ruined_portal_ocean")
    RUINED_PORTAL_NETHER = getStructure("ruined_portal_nether")
    ANCIENT_CITY = getStructure("ancient_city")
    TRAIL_RUINS = getStructure("trail_ruins")
    TRIAL_CHAMBERS = getStructure("trial_chambers")


    def getStructureType(self) -> "StructureType":
        """
        Returns the type of the structure.

        Returns
        - the type of structure
        """
        ...


    def getKey(self) -> "NamespacedKey":
        """
        See
        - .isRegistered()

        Deprecated
        - A key might not always be present, use .getKeyOrThrow() instead.
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.entity.EntityPickupItemEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import Item
from org.bukkit.entity import LivingEntity
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.entity import *
from typing import Any, Callable, Iterable, Tuple


class EntityPickupItemEvent(EntityEvent, Cancellable):
    """
    Thrown when a entity picks an item up from the ground
    """

    def __init__(self, entity: "LivingEntity", item: "Item", remaining: int):
        ...


    def getEntity(self) -> "LivingEntity":
        ...


    def getItem(self) -> "Item":
        """
        Gets the Item picked up by the entity.

        Returns
        - Item
        """
        ...


    def getRemaining(self) -> int:
        """
        Gets the amount remaining on the ground, if any

        Returns
        - amount remaining on the ground
        """
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file com.google.common.math.Quantiles

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.math import *
from com.google.common.primitives import Doubles
from com.google.common.primitives import Ints
from java.math import RoundingMode
from typing import Any, Callable, Iterable, Tuple


class Quantiles:
    """
    Provides a fluent API for calculating <a
    href="http://en.wikipedia.org/wiki/Quantile">quantiles</a>.
    
    <h3>Examples</h3>
    
    To compute the median:
    
    ````double myMedian = median().compute(myDataset);````
    
    where .median() has been statically imported.
    
    To compute the 99th percentile:
    
    ````double myPercentile99 = percentiles().index(99).compute(myDataset);````
    
    where .percentiles() has been statically imported.
    
    To compute median and the 90th and 99th percentiles:
    
    ````Map<Integer, Double> myPercentiles =
        percentiles().indexes(50, 90, 99).compute(myDataset);````
    
    where .percentiles() has been statically imported: `myPercentiles` maps the keys
    50, 90, and 99, to their corresponding quantile values.
    
    To compute quartiles, use .quartiles() instead of .percentiles(). To compute
    arbitrary q-quantiles, use .scale scale(q).
    
    These examples all take a copy of your dataset. If you have a double array, you are okay with
    it being arbitrarily reordered, and you want to avoid that copy, you can use `computeInPlace` instead of `compute`.
    
    <h3>Definition and notes on interpolation</h3>
    
    The definition of the kth q-quantile of N values is as follows: define x = k * (N - 1) / q; if
    x is an integer, the result is the value which would appear at index x in the sorted dataset
    (unless there are Double.NaN NaN values, see below); otherwise, the result is the average
    of the values which would appear at the indexes floor(x) and ceil(x) weighted by (1-frac(x)) and
    frac(x) respectively. This is the same definition as used by Excel and by S, it is the Type 7
    definition in <a
    href="http://stat.ethz.ch/R-manual/R-devel/library/stats/html/quantile.html">R</a>, and it is
    described by <a
    href="http://en.wikipedia.org/wiki/Quantile#Estimating_the_quantiles_of_a_population">
    wikipedia</a> as providing "Linear interpolation of the modes for the order statistics for the
    uniform distribution on [0,1]."
    
    <h3>Handling of non-finite values</h3>
    
    If any values in the input are Double.NaN NaN then all values returned are Double.NaN NaN. (This is the one occasion when the behaviour is not the same as you'd get from
    sorting with java.util.Arrays.sort(double[]) Arrays.sort(double[]) or java.util.Collections.sort(java.util.List) Collections.sort(List&lt;Double&gt;) and selecting
    the required value(s). Those methods would sort Double.NaN NaN as if it is greater than
    any other value and place them at the end of the dataset, even after Double.POSITIVE_INFINITY POSITIVE_INFINITY.)
    
    Otherwise, Double.NEGATIVE_INFINITY NEGATIVE_INFINITY and Double.POSITIVE_INFINITY POSITIVE_INFINITY sort to the beginning and the end of the dataset, as
    you would expect.
    
    If required to do a weighted average between an infinity and a finite value, or between an
    infinite value and itself, the infinite value is returned. If required to do a weighted average
    between Double.NEGATIVE_INFINITY NEGATIVE_INFINITY and Double.POSITIVE_INFINITY
    POSITIVE_INFINITY, Double.NaN NaN is returned (note that this will only happen if the
    dataset contains no finite values).
    
    <h3>Performance</h3>
    
    The average time complexity of the computation is O(N) in the size of the dataset. There is a
    worst case time complexity of O(N^2). You are extremely unlikely to hit this quadratic case on
    randomly ordered data (the probability decreases faster than exponentially in N), but if you are
    passing in unsanitized user data then a malicious user could force it. A light shuffle of the
    data using an unpredictable seed should normally be enough to thwart this attack.
    
    The time taken to compute multiple quantiles on the same dataset using Scale.indexes
    indexes is generally less than the total time taken to compute each of them separately, and
    sometimes much less. For example, on a large enough dataset, computing the 90th and 99th
    percentiles together takes about 55% as long as computing them separately.
    
    When calling ScaleAndIndex.compute (in ScaleAndIndexes.compute either
    form), the memory requirement is 8*N bytes for the copy of the dataset plus an overhead which is
    independent of N (but depends on the quantiles being computed). When calling ScaleAndIndex.computeInPlace computeInPlace (in ScaleAndIndexes.computeInPlace
    either form), only the overhead is required. The number of object allocations is independent of
    N in both cases.

    Author(s)
    - Pete Gillin

    Since
    - 20.0
    """

    @staticmethod
    def median() -> "ScaleAndIndex":
        """
        Specifies the computation of a median (i.e. the 1st 2-quantile).
        """
        ...


    @staticmethod
    def quartiles() -> "Scale":
        """
        Specifies the computation of quartiles (i.e. 4-quantiles).
        """
        ...


    @staticmethod
    def percentiles() -> "Scale":
        """
        Specifies the computation of percentiles (i.e. 100-quantiles).
        """
        ...


    @staticmethod
    def scale(scale: int) -> "Scale":
        """
        Specifies the computation of q-quantiles.

        Arguments
        - scale: the scale for the quantiles to be calculated, i.e. the q of the q-quantiles, which
            must be positive
        """
        ...


    class Scale:
        """
        Describes the point in a fluent API chain where only the scale (i.e. the q in q-quantiles) has
        been specified.

        Since
        - 20.0
        """

        def index(self, index: int) -> "ScaleAndIndex":
            """
            Specifies a single quantile index to be calculated, i.e. the k in the kth q-quantile.

            Arguments
            - index: the quantile index, which must be in the inclusive range [0, q] for q-quantiles
            """
            ...


        def indexes(self, *indexes: Tuple[int, ...]) -> "ScaleAndIndexes":
            """
            Specifies multiple quantile indexes to be calculated, each index being the k in the kth
            q-quantile.

            Arguments
            - indexes: the quantile indexes, each of which must be in the inclusive range [0, q] for
                q-quantiles; the order of the indexes is unimportant, duplicates will be ignored, and the
                set will be snapshotted when this method is called

            Raises
            - IllegalArgumentException: if `indexes` is empty
            """
            ...


        def indexes(self, indexes: Iterable["Integer"]) -> "ScaleAndIndexes":
            """
            Specifies multiple quantile indexes to be calculated, each index being the k in the kth
            q-quantile.

            Arguments
            - indexes: the quantile indexes, each of which must be in the inclusive range [0, q] for
                q-quantiles; the order of the indexes is unimportant, duplicates will be ignored, and the
                set will be snapshotted when this method is called

            Raises
            - IllegalArgumentException: if `indexes` is empty
            """
            ...


    class ScaleAndIndex:
        """
        Describes the point in a fluent API chain where the scale and a single quantile index (i.e. the
        q and the k in the kth q-quantile) have been specified.

        Since
        - 20.0
        """

        def compute(self, dataset: Iterable["Number"]) -> float:
            """
            Computes the quantile value of the given dataset.

            Arguments
            - dataset: the dataset to do the calculation on, which must be non-empty, which will be
                cast to doubles (with any associated lost of precision), and which will not be mutated by
                this call (it is copied instead)

            Returns
            - the quantile value
            """
            ...


        def compute(self, *dataset: Tuple[float, ...]) -> float:
            """
            Computes the quantile value of the given dataset.

            Arguments
            - dataset: the dataset to do the calculation on, which must be non-empty, which will not
                be mutated by this call (it is copied instead)

            Returns
            - the quantile value
            """
            ...


        def compute(self, *dataset: Tuple[int, ...]) -> float:
            """
            Computes the quantile value of the given dataset.

            Arguments
            - dataset: the dataset to do the calculation on, which must be non-empty, which will be
                cast to doubles (with any associated lost of precision), and which will not be mutated by
                this call (it is copied instead)

            Returns
            - the quantile value
            """
            ...


        def compute(self, *dataset: Tuple[int, ...]) -> float:
            """
            Computes the quantile value of the given dataset.

            Arguments
            - dataset: the dataset to do the calculation on, which must be non-empty, which will be
                cast to doubles, and which will not be mutated by this call (it is copied instead)

            Returns
            - the quantile value
            """
            ...


        def computeInPlace(self, *dataset: Tuple[float, ...]) -> float:
            """
            Computes the quantile value of the given dataset, performing the computation in-place.

            Arguments
            - dataset: the dataset to do the calculation on, which must be non-empty, and which will
                be arbitrarily reordered by this method call

            Returns
            - the quantile value
            """
            ...


    class ScaleAndIndexes:
        """
        Describes the point in a fluent API chain where the scale and a multiple quantile indexes (i.e.
        the q and a set of values for the k in the kth q-quantile) have been specified.

        Since
        - 20.0
        """

        def compute(self, dataset: Iterable["Number"]) -> dict["Integer", "Double"]:
            """
            Computes the quantile values of the given dataset.

            Arguments
            - dataset: the dataset to do the calculation on, which must be non-empty, which will be
                cast to doubles (with any associated lost of precision), and which will not be mutated by
                this call (it is copied instead)

            Returns
            - an unmodifiable, ordered map of results: the keys will be the specified quantile
                indexes, and the values the corresponding quantile values. When iterating, entries in the
                map are ordered by quantile index in the same order they were passed to the `indexes` method.
            """
            ...


        def compute(self, *dataset: Tuple[float, ...]) -> dict["Integer", "Double"]:
            """
            Computes the quantile values of the given dataset.

            Arguments
            - dataset: the dataset to do the calculation on, which must be non-empty, which will not
                be mutated by this call (it is copied instead)

            Returns
            - an unmodifiable, ordered map of results: the keys will be the specified quantile
                indexes, and the values the corresponding quantile values. When iterating, entries in the
                map are ordered by quantile index in the same order they were passed to the `indexes` method.
            """
            ...


        def compute(self, *dataset: Tuple[int, ...]) -> dict["Integer", "Double"]:
            """
            Computes the quantile values of the given dataset.

            Arguments
            - dataset: the dataset to do the calculation on, which must be non-empty, which will be
                cast to doubles (with any associated lost of precision), and which will not be mutated by
                this call (it is copied instead)

            Returns
            - an unmodifiable, ordered map of results: the keys will be the specified quantile
                indexes, and the values the corresponding quantile values. When iterating, entries in the
                map are ordered by quantile index in the same order they were passed to the `indexes` method.
            """
            ...


        def compute(self, *dataset: Tuple[int, ...]) -> dict["Integer", "Double"]:
            """
            Computes the quantile values of the given dataset.

            Arguments
            - dataset: the dataset to do the calculation on, which must be non-empty, which will be
                cast to doubles, and which will not be mutated by this call (it is copied instead)

            Returns
            - an unmodifiable, ordered map of results: the keys will be the specified quantile
                indexes, and the values the corresponding quantile values. When iterating, entries in the
                map are ordered by quantile index in the same order they were passed to the `indexes` method.
            """
            ...


        def computeInPlace(self, *dataset: Tuple[float, ...]) -> dict["Integer", "Double"]:
            """
            Computes the quantile values of the given dataset, performing the computation in-place.

            Arguments
            - dataset: the dataset to do the calculation on, which must be non-empty, and which will
                be arbitrarily reordered by this method call

            Returns
            - an unmodifiable, ordered map of results: the keys will be the specified quantile
                indexes, and the values the corresponding quantile values. When iterating, entries in the
                map are ordered by quantile index in the same order that the indexes were passed to the
                `indexes` method.
            """
            ...



"""
Python module generated from Java source file com.google.common.util.concurrent.FuturesGetChecked

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.annotations import VisibleForTesting
from com.google.common.collect import Ordering
from com.google.common.util.concurrent import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.j2objc.annotations import J2ObjCIncompatible
from java.lang.ref import WeakReference
from java.lang.reflect import Constructor
from java.lang.reflect import InvocationTargetException
from java.util import Arrays
from java.util.concurrent import CopyOnWriteArraySet
from java.util.concurrent import ExecutionException
from java.util.concurrent import Future
from java.util.concurrent import TimeUnit
from java.util.concurrent import TimeoutException
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class FuturesGetChecked:
    """
    Static methods used to implement Futures.getChecked(Future, Class).
    """





"""
Python module generated from Java source file org.bukkit.entity.minecart.StorageMinecart

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import Minecart
from org.bukkit.entity.minecart import *
from org.bukkit.inventory import InventoryHolder
from org.bukkit.loot import Lootable
from typing import Any, Callable, Iterable, Tuple


class StorageMinecart(Minecart, InventoryHolder, Lootable):
    """
    Represents a minecart with a chest. These types of Minecart
    minecarts have their own inventory that can be accessed using methods
    from the InventoryHolder interface.
    """


